# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2024/4/14 15:00
Desc: 东方财富网-数据中心-沪深港通持股
https://data.eastmoney.com/hsgtcg/
沪深港通详情: https://finance.eastmoney.com/news/1622,20161118685370149.html
"""

import pandas as pd
import requests
from bs4 import BeautifulSoup
from tqdm import tqdm


def stock_hsgt_fund_flow_summary_em() -> pd.DataFrame:
    """
    东方财富网-数据中心-资金流向-沪深港通资金流向
    https://data.eastmoney.com/hsgt/index.html#lssj
    :return: 沪深港通资金流向
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_MUTUAL_QUOTA",
        "columns": "TRADE_DATE,MUTUAL_TYPE,BOARD_TYPE,MUTUAL_TYPE_NAME,FUNDS_DIRECTION,"
        "INDEX_CODE,INDEX_NAME,BOARD_CODE",
        "quoteColumns": "status~07~BOARD_CODE,dayNetAmtIn~07~BOARD_CODE,dayAmtRemain~07~BOARD_CODE,"
        "dayAmtThreshold~07~BOARD_CODE,f104~07~BOARD_CODE,f105~07~BOARD_CODE,"
        "f106~07~BOARD_CODE,f3~03~INDEX_CODE~INDEX_f3,netBuyAmt~07~BOARD_CODE",
        "quoteType": "0",
        "pageNumber": "1",
        "pageSize": "2000",
        "sortTypes": "1",
        "sortColumns": "MUTUAL_TYPE",
        "source": "WEB",
        "client": "WEB",
        "_": "1669047266881",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.columns = [
        "交易日",
        "-",
        "类型",
        "板块",
        "资金方向",
        "-",
        "相关指数",
        "-",
        "交易状态",
        "资金净流入",
        "当日资金余额",
        "-",
        "上涨数",
        "下跌数",
        "持平数",
        "指数涨跌幅",
        "成交净买额",
    ]
    temp_df = temp_df[
        [
            "交易日",
            "类型",
            "板块",
            "资金方向",
            "交易状态",
            "成交净买额",
            "资金净流入",
            "当日资金余额",
            "上涨数",
            "持平数",
            "下跌数",
            "相关指数",
            "指数涨跌幅",
        ]
    ]
    temp_df["交易日"] = pd.to_datetime(temp_df["交易日"], errors="coerce").dt.date
    temp_df["成交净买额"] = pd.to_numeric(temp_df["成交净买额"], errors="coerce")
    temp_df["资金净流入"] = pd.to_numeric(temp_df["资金净流入"], errors="coerce")
    temp_df["当日资金余额"] = pd.to_numeric(temp_df["当日资金余额"], errors="coerce")
    temp_df["上涨数"] = pd.to_numeric(temp_df["上涨数"], errors="coerce")
    temp_df["持平数"] = pd.to_numeric(temp_df["持平数"], errors="coerce")
    temp_df["下跌数"] = pd.to_numeric(temp_df["下跌数"], errors="coerce")
    temp_df["指数涨跌幅"] = pd.to_numeric(temp_df["指数涨跌幅"], errors="coerce")
    temp_df["成交净买额"] = temp_df["成交净买额"] / 10000
    temp_df["资金净流入"] = temp_df["资金净流入"] / 10000
    temp_df["当日资金余额"] = temp_df["当日资金余额"] / 10000
    return temp_df


def stock_hk_ggt_components_em() -> pd.DataFrame:
    """
    东方财富网-行情中心-港股市场-港股通成份股
    https://quote.eastmoney.com/center/gridlist.html#hk_components
    :return: 港股通成份股
    :rtype: pandas.DataFrame
    """
    url = "https://33.push2.eastmoney.com/api/qt/clist/get"
    params = {
        "pn": "1",
        "pz": "5000",
        "po": "1",
        "np": "1",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "fid": "f3",
        "fs": "b:DLMK0146,b:DLMK0144",
        "fields": "f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f12,f13,f14,f15,f16,f17,f18,f19,f20,f21,f23,f24,"
        "f25,f26,f22,f33,f11,f62,f128,f136,f115,f152",
        "_": "1639974456250",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"]["diff"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = temp_df.index + 1
    temp_df.columns = [
        "序号",
        "-",
        "最新价",
        "涨跌幅",
        "涨跌额",
        "成交量",
        "成交额",
        "-",
        "-",
        "-",
        "-",
        "-",
        "代码",
        "-",
        "名称",
        "最高",
        "最低",
        "今开",
        "昨收",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
    ]
    temp_df = temp_df[
        [
            "序号",
            "代码",
            "名称",
            "最新价",
            "涨跌额",
            "涨跌幅",
            "今开",
            "最高",
            "最低",
            "昨收",
            "成交量",
            "成交额",
        ]
    ]
    return temp_df


def stock_hsgt_hold_stock_em(
    market: str = "沪股通", indicator: str = "5日排行"
) -> pd.DataFrame:
    """
    东方财富-数据中心-沪深港通持股-个股排行
    https://data.eastmoney.com/hsgtcg/list.html
    :param market: choice of {"北向", "沪股通", "深股通"}
    :type market: str
    :param indicator: choice of {"今日排行", "3日排行", "5日排行", "10日排行", "月排行", "季排行", "年排行"}
    :type indicator: str
    :return: 指定 sector 和 indicator 的数据
    :rtype: pandas.DataFrame
    """
    url = "https://data.eastmoney.com/hsgtcg/list.html"
    r = requests.get(url)
    soup = BeautifulSoup(r.text, "lxml")
    date = (
        soup.find("div", attrs={"class": "title"})
        .find("span")
        .text.strip("（")
        .strip("）")
    )
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    indicator_type = ""
    filter_str = ""
    if indicator == "今日排行":
        indicator_type = "1"
    if indicator == "3日排行":
        indicator_type = "3"
    if indicator == "5日排行":
        indicator_type = "5"
    if indicator == "10日排行":
        indicator_type = "10"
    if indicator == "月排行":
        indicator_type = "M"
    if indicator == "季排行":
        indicator_type = "Q"
    if indicator == "年排行":
        indicator_type = "Y"
    if market == "北向":
        filter_str = f"""(TRADE_DATE='{date}')(INTERVAL_TYPE="{indicator_type}")"""
    elif market == "沪股通":
        filter_str = f"""(TRADE_DATE='{date}')(INTERVAL_TYPE="{indicator_type}")(MUTUAL_TYPE="001")"""
    elif market == "深股通":
        filter_str = f"""(TRADE_DATE='{date}')(INTERVAL_TYPE="{indicator_type}")(MUTUAL_TYPE="003")"""
    params = {
        "sortColumns": "ADD_MARKET_CAP",
        "sortTypes": "-1",
        "pageSize": "50000",
        "pageNumber": "1",
        "reportName": "RPT_MUTUAL_STOCK_NORTHSTA",
        "columns": "ALL",
        "source": "WEB",
        "client": "WEB",
        "filter": filter_str,
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    page_num = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    for page in tqdm(range(1, page_num + 1), leave=False):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat([big_df, temp_df], ignore_index=True)

    big_df.reset_index(inplace=True)
    big_df["index"] = range(1, len(big_df) + 1)
    big_df.columns = [
        "序号",
        "_",
        "_",
        "日期",
        "_",
        "名称",
        "_",
        "_",
        "代码",
        "_",
        "_",
        "_",
        "_",
        "今日持股-股数",
        "今日持股-市值",
        "今日持股-占流通股比",
        "今日持股-占总股本比",
        "今日收盘价",
        "今日涨跌幅",
        "_",
        "所属板块",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        f'{indicator.split("排")[0]}增持估计-市值',
        f'{indicator.split("排")[0]}增持估计-股数',
        f'{indicator.split("排")[0]}增持估计-市值增幅',
        f'{indicator.split("排")[0]}增持估计-占流通股比',
        f'{indicator.split("排")[0]}增持估计-占总股本比',
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]
    big_df = big_df[
        [
            "序号",
            "代码",
            "名称",
            "今日收盘价",
            "今日涨跌幅",
            "今日持股-股数",
            "今日持股-市值",
            "今日持股-占流通股比",
            "今日持股-占总股本比",
            f'{indicator.split("排")[0]}增持估计-股数',
            f'{indicator.split("排")[0]}增持估计-市值',
            f'{indicator.split("排")[0]}增持估计-市值增幅',
            f'{indicator.split("排")[0]}增持估计-占流通股比',
            f'{indicator.split("排")[0]}增持估计-占总股本比',
            "所属板块",
            "日期",
        ]
    ]
    big_df["今日收盘价"] = pd.to_numeric(big_df["今日收盘价"], errors="coerce")
    big_df["今日涨跌幅"] = pd.to_numeric(big_df["今日涨跌幅"], errors="coerce")
    big_df["今日持股-股数"] = pd.to_numeric(big_df["今日持股-股数"], errors="coerce")
    big_df["今日持股-市值"] = pd.to_numeric(big_df["今日持股-市值"], errors="coerce")
    big_df["今日持股-占流通股比"] = pd.to_numeric(
        big_df["今日持股-占流通股比"], errors="coerce"
    )
    big_df["今日持股-占总股本比"] = pd.to_numeric(
        big_df["今日持股-占总股本比"], errors="coerce"
    )
    big_df[f'{indicator.split("排")[0]}增持估计-股数'] = pd.to_numeric(
        big_df[f'{indicator.split("排")[0]}增持估计-股数'], errors="coerce"
    )
    big_df[f'{indicator.split("排")[0]}增持估计-市值'] = pd.to_numeric(
        big_df[f'{indicator.split("排")[0]}增持估计-市值'], errors="coerce"
    )
    big_df[f'{indicator.split("排")[0]}增持估计-市值增幅'] = pd.to_numeric(
        big_df[f'{indicator.split("排")[0]}增持估计-市值增幅'], errors="coerce"
    )
    big_df[f'{indicator.split("排")[0]}增持估计-占流通股比'] = pd.to_numeric(
        big_df[f'{indicator.split("排")[0]}增持估计-占流通股比'], errors="coerce"
    )
    big_df[f'{indicator.split("排")[0]}增持估计-占总股本比'] = pd.to_numeric(
        big_df[f'{indicator.split("排")[0]}增持估计-占总股本比'], errors="coerce"
    )
    big_df["日期"] = pd.to_datetime(big_df["日期"], errors="coerce").dt.date
    return big_df


def stock_hsgt_stock_statistics_em(
    symbol: str = "北向持股",
    start_date: str = "20240110",
    end_date: str = "20240110",
):
    """
    东方财富网-数据中心-沪深港通-沪深港通持股-每日个股统计
    https://data.eastmoney.com/hsgtcg/StockStatistics.aspx
    market=001, 沪股通持股
    market=003, 深股通持股
    :param symbol: choice of {"北向持股", "南向持股"}
    :type symbol: str
    :param start_date: 指定数据获取开始的时间, e.g., "20200713"
    :type start_date: str
    :param end_date: 指定数据获取结束的时间, e.g., "20200715"
    :type end_date:str
    :return: 指定市场和指定时间段的每日个股统计数据
    :rtype: pandas.DataFrame
    """
    start_date = "-".join([start_date[:4], start_date[4:6], start_date[6:]])
    end_date = "-".join([end_date[:4], end_date[4:6], end_date[6:]])
    big_df = pd.DataFrame()
    if symbol == "南向持股":
        params = {
            "sortColumns": "TRADE_DATE",
            "sortTypes": "-1",
            "pageSize": "1000",
            "pageNumber": "1",
            "columns": "ALL",
            "source": "WEB",
            "client": "WEB",
            "filter": f"""(INTERVAL_TYPE="1")(RN=1)(TRADE_DATE>='{start_date}')(TRADE_DATE<='{end_date}')""",
            "rt": "53160469",
            "reportName": "RPT_MUTUAL_STOCK_HOLDRANKS",
        }
        if start_date == end_date:
            params.update(
                {"filter": f"""(INTERVAL_TYPE="1")(RN=1)(TRADE_DATE='{start_date}')"""}
            )
        url = "http://datacenter-web.eastmoney.com/api/data/v1/get"
        r = requests.get(url, params=params)
        data_json = r.json()
        total_page = data_json["result"]["pages"]
        big_df = pd.DataFrame()
        for page in tqdm(range(1, int(total_page) + 1), leave=False):
            params.update({"pageNumber": page})
            r = requests.get(url, params=params)
            data_json = r.json()
            temp_df = pd.DataFrame(data_json["result"]["data"])
            big_df = pd.concat([big_df, temp_df], ignore_index=True)

        big_df.columns = [
            "-",
            "持股日期",
            "-",
            "-",
            "股票简称",
            "股票代码",
            "-",
            "-",
            "-",
            "-",
            "持股数量",
            "持股市值",
            "-",
            "-",
            "-",
            "-",
            "当日收盘价",
            "当日涨跌幅",
            "-",
            "-",
            "-",
            "-",
            "-",
            "持股数量占发行股百分比",
            "-",
            "持股市值变化-1日",
            "持股市值变化-5日",
            "持股市值变化-10日",
            "-",
        ]
        big_df = big_df[
            [
                "持股日期",
                "股票代码",
                "股票简称",
                "当日收盘价",
                "当日涨跌幅",
                "持股数量",
                "持股市值",
                "持股数量占发行股百分比",
                "持股市值变化-1日",
                "持股市值变化-5日",
                "持股市值变化-10日",
            ]
        ]
        big_df["持股日期"] = pd.to_datetime(big_df["持股日期"], errors="coerce").dt.date
        big_df["当日收盘价"] = pd.to_numeric(big_df["当日收盘价"], errors="coerce")
        big_df["当日涨跌幅"] = pd.to_numeric(big_df["当日涨跌幅"], errors="coerce")
        big_df["持股数量"] = pd.to_numeric(big_df["持股数量"], errors="coerce")
        big_df["持股市值"] = pd.to_numeric(big_df["持股市值"], errors="coerce")
        big_df["持股数量占发行股百分比"] = pd.to_numeric(
            big_df["持股数量占发行股百分比"], errors="coerce"
        )
        big_df["持股市值变化-1日"] = pd.to_numeric(
            big_df["持股市值变化-1日"], errors="coerce"
        )
        big_df["持股市值变化-5日"] = pd.to_numeric(
            big_df["持股市值变化-5日"], errors="coerce"
        )
        big_df["持股市值变化-10日"] = pd.to_numeric(
            big_df["持股市值变化-10日"], errors="coerce"
        )
    elif symbol == "北向持股":
        params = {
            "sortColumns": "TRADE_DATE",
            "sortTypes": "-1",
            "pageSize": "1000",
            "pageNumber": "1",
            "columns": "ALL",
            "source": "WEB",
            "client": "WEB",
            "filter": f"""(INTERVAL_TYPE="1")(MUTUAL_TYPE in ("001","003"))(TRADE_DATE>='{start_date}')(TRADE_DATE<='{end_date}')""",
            "rt": "53160469",
            "reportName": "RPT_MUTUAL_STOCK_NORTHSTA",
        }
        if start_date == end_date:
            params.update(
                {
                    "filter": f"""(INTERVAL_TYPE="1")(MUTUAL_TYPE in ("001","003"))(TRADE_DATE='{start_date}')"""
                }
            )
        url = "http://datacenter-web.eastmoney.com/api/data/v1/get"
        r = requests.get(url, params=params)
        data_json = r.json()
        total_page = data_json["result"]["pages"]
        big_df = pd.DataFrame()
        for page in tqdm(range(1, int(total_page) + 1), leave=False):
            params.update({"pageNumber": page})
            r = requests.get(url, params=params)
            data_json = r.json()
            temp_df = pd.DataFrame(data_json["result"]["data"])
            big_df = pd.concat([big_df, temp_df], ignore_index=True)

        big_df.columns = [
            "-",
            "-",
            "持股日期",
            "-",
            "股票简称",
            "-",
            "-",
            "股票代码",
            "-",
            "-",
            "-",
            "-",
            "持股数量",
            "持股市值",
            "-",
            "持股数量占发行股百分比",
            "当日收盘价",
            "当日涨跌幅",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "持股市值变化-1日",
            "持股市值变化-5日",
            "持股市值变化-10日",
            "-",
            "-",
            "-",
        ]
        big_df = big_df[
            [
                "持股日期",
                "股票代码",
                "股票简称",
                "当日收盘价",
                "当日涨跌幅",
                "持股数量",
                "持股市值",
                "持股数量占发行股百分比",
                "持股市值变化-1日",
                "持股市值变化-5日",
                "持股市值变化-10日",
            ]
        ]
        big_df["持股日期"] = pd.to_datetime(big_df["持股日期"], errors="coerce").dt.date
        big_df["当日收盘价"] = pd.to_numeric(big_df["当日收盘价"], errors="coerce")
        big_df["当日涨跌幅"] = pd.to_numeric(big_df["当日涨跌幅"], errors="coerce")
        big_df["持股数量"] = pd.to_numeric(big_df["持股数量"], errors="coerce")
        big_df["持股市值"] = pd.to_numeric(big_df["持股市值"], errors="coerce")
        big_df["持股数量占发行股百分比"] = pd.to_numeric(
            big_df["持股数量占发行股百分比"], errors="coerce"
        )
        big_df["持股市值变化-1日"] = pd.to_numeric(
            big_df["持股市值变化-1日"], errors="coerce"
        )
        big_df["持股市值变化-5日"] = pd.to_numeric(
            big_df["持股市值变化-5日"], errors="coerce"
        )
        big_df["持股市值变化-10日"] = pd.to_numeric(
            big_df["持股市值变化-10日"], errors="coerce"
        )
    elif symbol == "沪股通持股":
        params = {
            "sortColumns": "TRADE_DATE",
            "sortTypes": "-1",
            "pageSize": "1000",
            "pageNumber": "1",
            "columns": "ALL",
            "source": "WEB",
            "client": "WEB",
            "filter": f"""(INTERVAL_TYPE="1")(MUTUAL_TYPE="001")(TRADE_DATE>='{start_date}')(TRADE_DATE<='{end_date}')""",
            "reportName": "RPT_MUTUAL_STOCK_NORTHSTA",
        }
        if start_date == end_date:
            params.update(
                {
                    "filter": f"""(INTERVAL_TYPE="1")(MUTUAL_TYPE="001")(TRADE_DATE='{start_date}')"""
                }
            )
        url = "http://datacenter-web.eastmoney.com/api/data/v1/get"
        r = requests.get(url, params=params)
        data_json = r.json()
        total_page = data_json["result"]["pages"]
        big_df = pd.DataFrame()
        for page in tqdm(range(1, int(total_page) + 1), leave=False):
            params.update({"pageNumber": page})
            r = requests.get(url, params=params)
            data_json = r.json()
            temp_df = pd.DataFrame(data_json["result"]["data"])
            big_df = pd.concat([big_df, temp_df], ignore_index=True)
        big_df.columns = [
            "-",
            "-",
            "持股日期",
            "-",
            "股票简称",
            "-",
            "-",
            "股票代码",
            "-",
            "-",
            "-",
            "-",
            "持股数量",
            "持股市值",
            "-",
            "持股数量占发行股百分比",
            "当日收盘价",
            "当日涨跌幅",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "持股市值变化-1日",
            "持股市值变化-5日",
            "持股市值变化-10日",
            "-",
            "-",
            "-",
        ]
        big_df = big_df[
            [
                "持股日期",
                "股票代码",
                "股票简称",
                "当日收盘价",
                "当日涨跌幅",
                "持股数量",
                "持股市值",
                "持股数量占发行股百分比",
                "持股市值变化-1日",
                "持股市值变化-5日",
                "持股市值变化-10日",
            ]
        ]
        big_df["持股日期"] = pd.to_datetime(big_df["持股日期"], errors="coerce").dt.date
        big_df["当日收盘价"] = pd.to_numeric(big_df["当日收盘价"], errors="coerce")
        big_df["当日涨跌幅"] = pd.to_numeric(big_df["当日涨跌幅"], errors="coerce")
        big_df["持股数量"] = pd.to_numeric(big_df["持股数量"], errors="coerce")
        big_df["持股市值"] = pd.to_numeric(big_df["持股市值"], errors="coerce")
        big_df["持股数量占发行股百分比"] = pd.to_numeric(
            big_df["持股数量占发行股百分比"], errors="coerce"
        )
        big_df["持股市值变化-1日"] = pd.to_numeric(
            big_df["持股市值变化-1日"], errors="coerce"
        )
        big_df["持股市值变化-5日"] = pd.to_numeric(
            big_df["持股市值变化-5日"], errors="coerce"
        )
        big_df["持股市值变化-10日"] = pd.to_numeric(
            big_df["持股市值变化-10日"], errors="coerce"
        )
    elif symbol == "深股通持股":
        params = {
            "sortColumns": "TRADE_DATE",
            "sortTypes": "-1",
            "pageSize": "1000",
            "pageNumber": "1",
            "columns": "ALL",
            "source": "WEB",
            "client": "WEB",
            "filter": f"""(INTERVAL_TYPE="1")(MUTUAL_TYPE="003")(TRADE_DATE>='{start_date}')(TRADE_DATE<='{end_date}')""",
            "reportName": "RPT_MUTUAL_STOCK_NORTHSTA",
        }
        if start_date == end_date:
            params.update(
                {
                    "filter": f"""(INTERVAL_TYPE="1")(MUTUAL_TYPE="003")(TRADE_DATE='{start_date}')"""
                }
            )
        url = "http://datacenter-web.eastmoney.com/api/data/v1/get"
        r = requests.get(url, params=params)
        data_json = r.json()
        total_page = data_json["result"]["pages"]
        big_df = pd.DataFrame()
        for page in tqdm(range(1, int(total_page) + 1), leave=False):
            params.update({"pageNumber": page})
            r = requests.get(url, params=params)
            data_json = r.json()
            temp_df = pd.DataFrame(data_json["result"]["data"])
            big_df = pd.concat([big_df, temp_df], ignore_index=True)
        big_df.columns = [
            "-",
            "-",
            "持股日期",
            "-",
            "股票简称",
            "-",
            "-",
            "股票代码",
            "-",
            "-",
            "-",
            "-",
            "持股数量",
            "持股市值",
            "-",
            "持股数量占发行股百分比",
            "当日收盘价",
            "当日涨跌幅",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "持股市值变化-1日",
            "持股市值变化-5日",
            "持股市值变化-10日",
            "-",
            "-",
            "-",
        ]
        big_df = big_df[
            [
                "持股日期",
                "股票代码",
                "股票简称",
                "当日收盘价",
                "当日涨跌幅",
                "持股数量",
                "持股市值",
                "持股数量占发行股百分比",
                "持股市值变化-1日",
                "持股市值变化-5日",
                "持股市值变化-10日",
            ]
        ]
        big_df["持股日期"] = pd.to_datetime(big_df["持股日期"], errors="coerce").dt.date
        big_df["当日收盘价"] = pd.to_numeric(big_df["当日收盘价"], errors="coerce")
        big_df["当日涨跌幅"] = pd.to_numeric(big_df["当日涨跌幅"], errors="coerce")
        big_df["持股数量"] = pd.to_numeric(big_df["持股数量"], errors="coerce")
        big_df["持股市值"] = pd.to_numeric(big_df["持股市值"], errors="coerce")
        big_df["持股数量占发行股百分比"] = pd.to_numeric(
            big_df["持股数量占发行股百分比"], errors="coerce"
        )
        big_df["持股市值变化-1日"] = pd.to_numeric(
            big_df["持股市值变化-1日"], errors="coerce"
        )
        big_df["持股市值变化-5日"] = pd.to_numeric(
            big_df["持股市值变化-5日"], errors="coerce"
        )
        big_df["持股市值变化-10日"] = pd.to_numeric(
            big_df["持股市值变化-10日"], errors="coerce"
        )
    return big_df


def stock_hsgt_institution_statistics_em(
    market: str = "北向持股",
    start_date: str = "20220601",
    end_date: str = "20220609",
):
    """
    东方财富网-数据中心-沪深港通-沪深港通持股-每日机构统计
    https://data.eastmoney.com/hsgtcg/InstitutionStatistics.aspx
    :param market: choice of {"北向持股", "南向持股", "沪股通持股", "深股通持股"}
    :type market: str
    :param start_date: 指定数据获取开始的时间, e.g., "20200713"
    :type start_date: str
    :param end_date: 指定数据获取结束的时间, e.g., "20200715"
    :type end_date:str
    :return: 指定市场和指定时间段的每日个股统计数据
    :rtype: pandas.DataFrame
    """
    start_date = "-".join([start_date[:4], start_date[4:6], start_date[6:]])
    end_date = "-".join([end_date[:4], end_date[4:6], end_date[6:]])
    if market == "南向持股":
        params = {
            "sortColumns": "HOLD_DATE",
            "sortTypes": "-1",
            "pageSize": "500",
            "pageNumber": "1",
            "reportName": "PRT_MUTUAL_ORG_STA",
            "columns": "ALL",
            "source": "WEB",
            "client": "WEB",
            "filter": f"""(MARKET_TYPE="S")(HOLD_DATE>='{start_date}')(HOLD_DATE<='{end_date}')""",
        }
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.116 Safari/537.36"
        }
        url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
        r = requests.get(url, params=params, headers=headers)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        temp_df.columns = [
            "持股日期",
            "_",
            "持股只数",
            "_",
            "持股市值",
            "持股市值变化-1日",
            "持股市值变化-5日",
            "持股市值变化-10日",
            "_",
            "机构名称",
            "_",
            "_",
            "_",
            "_",
            "_",
            "_",
            "_",
            "_",
        ]
        temp_df = temp_df[
            [
                "持股日期",
                "机构名称",
                "持股只数",
                "持股市值",
                "持股市值变化-1日",
                "持股市值变化-5日",
                "持股市值变化-10日",
            ]
        ]
        temp_df["持股日期"] = pd.to_datetime(temp_df["持股日期"]).dt.date
        temp_df["持股只数"] = pd.to_numeric(temp_df["持股只数"])
        temp_df["持股市值"] = pd.to_numeric(temp_df["持股市值"])
        temp_df["持股市值变化-1日"] = pd.to_numeric(temp_df["持股市值变化-1日"])
        temp_df["持股市值变化-5日"] = pd.to_numeric(temp_df["持股市值变化-5日"])
        temp_df["持股市值变化-10日"] = pd.to_numeric(temp_df["持股市值变化-10日"])
        return temp_df

    elif market == "北向持股":
        params = {
            "sortColumns": "HOLD_DATE",
            "sortTypes": "-1",
            "pageSize": "500",
            "pageNumber": "1",
            "reportName": "PRT_MUTUAL_ORG_STA",
            "columns": "ALL",
            "source": "WEB",
            "client": "WEB",
            "filter": f"""(MARKET_TYPE="N")(HOLD_DATE>='{start_date}')(HOLD_DATE<='{end_date}')""",
        }
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.116 Safari/537.36"
        }
        url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
        r = requests.get(url, params=params, headers=headers)
        data_json = r.json()
        total_page = data_json["result"]["pages"]
        big_df = pd.DataFrame()
        for page in tqdm(range(1, total_page + 1), leave=False):
            params.update({"pageNumber": page})
            r = requests.get(url, params=params, headers=headers)
            data_json = r.json()
            temp_df = pd.DataFrame(data_json["result"]["data"])
            big_df = pd.concat([big_df, temp_df], ignore_index=True)
        big_df.columns = [
            "持股日期",
            "_",
            "持股只数",
            "_",
            "持股市值",
            "持股市值变化-1日",
            "持股市值变化-5日",
            "持股市值变化-10日",
            "_",
            "机构名称",
            "_",
            "_",
            "_",
            "_",
            "_",
            "_",
            "_",
            "_",
        ]
        big_df = big_df[
            [
                "持股日期",
                "机构名称",
                "持股只数",
                "持股市值",
                "持股市值变化-1日",
                "持股市值变化-5日",
                "持股市值变化-10日",
            ]
        ]
        big_df["持股日期"] = pd.to_datetime(big_df["持股日期"]).dt.date
        big_df["持股只数"] = pd.to_numeric(big_df["持股只数"])
        big_df["持股市值"] = pd.to_numeric(big_df["持股市值"])
        big_df["持股市值变化-1日"] = pd.to_numeric(big_df["持股市值变化-1日"])
        big_df["持股市值变化-5日"] = pd.to_numeric(big_df["持股市值变化-5日"])
        big_df["持股市值变化-10日"] = pd.to_numeric(big_df["持股市值变化-10日"])
        return big_df
    elif market == "沪股通持股":
        params = {
            "sortColumns": "HOLD_DATE",
            "sortTypes": "-1",
            "pageSize": "500",
            "pageNumber": "1",
            "reportName": "PRT_MUTUAL_ORG_STA",
            "columns": "ALL",
            "source": "WEB",
            "client": "WEB",
            "filter": f"""(MARKET_TYPE="001")(HOLD_DATE>='{start_date}')(HOLD_DATE<='{end_date}')""",
        }
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.116 Safari/537.36"
        }
        url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
        r = requests.get(url, params=params, headers=headers)
        data_json = r.json()
        total_page = data_json["result"]["pages"]
        big_df = pd.DataFrame()
        for page in tqdm(range(1, total_page + 1), leave=False):
            params.update({"pageNumber": page})
            r = requests.get(url, params=params, headers=headers)
            data_json = r.json()
            temp_df = pd.DataFrame(data_json["result"]["data"])
            big_df = pd.concat([big_df, temp_df], ignore_index=True)
        big_df.columns = [
            "持股日期",
            "_",
            "持股只数",
            "_",
            "持股市值",
            "持股市值变化-1日",
            "持股市值变化-5日",
            "持股市值变化-10日",
            "_",
            "机构名称",
            "_",
            "_",
            "_",
            "_",
            "_",
            "_",
            "_",
            "_",
        ]
        big_df = big_df[
            [
                "持股日期",
                "机构名称",
                "持股只数",
                "持股市值",
                "持股市值变化-1日",
                "持股市值变化-5日",
                "持股市值变化-10日",
            ]
        ]
        big_df["持股日期"] = pd.to_datetime(big_df["持股日期"]).dt.date
        big_df["持股只数"] = pd.to_numeric(big_df["持股只数"])
        big_df["持股市值"] = pd.to_numeric(big_df["持股市值"])
        big_df["持股市值变化-1日"] = pd.to_numeric(big_df["持股市值变化-1日"])
        big_df["持股市值变化-5日"] = pd.to_numeric(big_df["持股市值变化-5日"])
        big_df["持股市值变化-10日"] = pd.to_numeric(big_df["持股市值变化-10日"])
        return big_df
    elif market == "深股通持股":
        params = {
            "sortColumns": "HOLD_DATE",
            "sortTypes": "-1",
            "pageSize": "500",
            "pageNumber": "1",
            "reportName": "PRT_MUTUAL_ORG_STA",
            "columns": "ALL",
            "source": "WEB",
            "client": "WEB",
            "filter": f"""(MARKET_TYPE="003")(HOLD_DATE>='{start_date}')(HOLD_DATE<='{end_date}')""",
        }
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.116 Safari/537.36"
        }
        url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
        r = requests.get(url, params=params, headers=headers)
        data_json = r.json()
        total_page = data_json["result"]["pages"]
        big_df = pd.DataFrame()
        for page in tqdm(range(1, total_page + 1), leave=False):
            params.update({"pageNumber": page})
            r = requests.get(url, params=params, headers=headers)
            data_json = r.json()
            temp_df = pd.DataFrame(data_json["result"]["data"])
            big_df = pd.concat([big_df, temp_df], ignore_index=True)
        big_df.columns = [
            "持股日期",
            "_",
            "持股只数",
            "_",
            "持股市值",
            "持股市值变化-1日",
            "持股市值变化-5日",
            "持股市值变化-10日",
            "_",
            "机构名称",
            "_",
            "_",
            "_",
            "_",
            "_",
            "_",
            "_",
            "_",
        ]
        big_df = big_df[
            [
                "持股日期",
                "机构名称",
                "持股只数",
                "持股市值",
                "持股市值变化-1日",
                "持股市值变化-5日",
                "持股市值变化-10日",
            ]
        ]
        big_df["持股日期"] = pd.to_datetime(big_df["持股日期"]).dt.date
        big_df["持股只数"] = pd.to_numeric(big_df["持股只数"])
        big_df["持股市值"] = pd.to_numeric(big_df["持股市值"])
        big_df["持股市值变化-1日"] = pd.to_numeric(big_df["持股市值变化-1日"])
        big_df["持股市值变化-5日"] = pd.to_numeric(big_df["持股市值变化-5日"])
        big_df["持股市值变化-10日"] = pd.to_numeric(big_df["持股市值变化-10日"])
        return big_df


def stock_hsgt_hist_em(symbol: str = "北向资金") -> pd.DataFrame:
    """
    东方财富网-数据中心-资金流向-沪深港通资金流向-沪深港通历史数据
    https://data.eastmoney.com/hsgt/index.html
    :param symbol: choice of {"北向资金", "沪股通", "深股通", "南向资金", "港股通沪", "港股通深"}
    :type symbol: str
    :return: 沪深港通历史数据
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "北向资金": "5",
        "沪股通": "1",
        "深股通": "3",
        "南向资金": "6",
        "港股通沪": "2",
        "港股通深": "4",
    }
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "TRADE_DATE",
        "sortTypes": "-1",
        "pageSize": "1000",
        "pageNumber": "1",
        "reportName": "RPT_MUTUAL_DEAL_HISTORY",
        "columns": "ALL",
        "source": "WEB",
        "client": "WEB",
        "filter": f'(MUTUAL_TYPE="00{symbol_map[symbol]}")',
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    for page in tqdm(range(1, int(total_page) + 1), leave=False):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    if symbol == "北向资金":
        index_name = "沪深300"
    elif symbol == "沪股通":
        index_name = "上证指数"
    elif symbol == "深股通":
        index_name = "深证指数"
    elif symbol == "南向资金":
        index_name = "沪深300"
    else:
        index_name = "恒生指数"

    big_df.rename(
        columns={
            "MUTUAL_TYPE": "-",
            "TRADE_DATE": "日期",
            "FUND_INFLOW": "当日资金流入",
            "NET_DEAL_AMT": "当日成交净买额",
            "QUOTA_BALANCE": "当日余额",
            "ACCUM_DEAL_AMT": "历史累计净买额",
            "BUY_AMT": "买入成交额",
            "SELL_AMT": "卖出成交额",
            "LEAD_STOCKS_CODE": "领涨股-代码",
            "LEAD_STOCKS_NAME": "领涨股",
            "LS_CHANGE_RATE": "领涨股-涨跌幅",
            "INDEX_CLOSE_PRICE": index_name,
            "INDEX_CHANGE_RATE": f"{index_name}-涨跌幅",
            "HOLD_MARKET_CAP": "持股市值",
        },
        inplace=True,
    )
    big_df = big_df[
        [
            "日期",
            "当日成交净买额",
            "买入成交额",
            "卖出成交额",
            "历史累计净买额",
            "当日资金流入",
            "当日余额",
            "持股市值",
            "领涨股",
            "领涨股-涨跌幅",
            index_name,
            f"{index_name}-涨跌幅",
            "领涨股-代码",
        ]
    ]
    big_df["日期"] = pd.to_datetime(big_df["日期"], errors="coerce").dt.date
    big_df.sort_values(["日期"], inplace=True, ignore_index=True)
    big_df["当日资金流入"] = (
        pd.to_numeric(big_df["当日资金流入"], errors="coerce") / 100
    )
    big_df["当日余额"] = pd.to_numeric(big_df["当日余额"], errors="coerce") / 100
    if symbol == "沪股通" or symbol == "深股通":
        big_df["历史累计净买额"] = (
            pd.to_numeric(big_df["历史累计净买额"], errors="coerce") / 100
        )
    else:
        big_df["历史累计净买额"] = (
            pd.to_numeric(big_df["历史累计净买额"], errors="coerce") / 100 / 10000
        )
    big_df["当日成交净买额"] = (
        pd.to_numeric(big_df["当日成交净买额"], errors="coerce") / 100
    )
    big_df["买入成交额"] = pd.to_numeric(big_df["买入成交额"], errors="coerce") / 100
    big_df["卖出成交额"] = pd.to_numeric(big_df["卖出成交额"], errors="coerce") / 100
    big_df["领涨股-涨跌幅"] = pd.to_numeric(big_df["领涨股-涨跌幅"], errors="coerce")
    big_df[index_name] = pd.to_numeric(big_df[index_name], errors="coerce")
    big_df[f"{index_name}-涨跌幅"] = pd.to_numeric(
        big_df[f"{index_name}-涨跌幅"], errors="coerce"
    )
    big_df["持股市值"] = pd.to_numeric(big_df["持股市值"], errors="coerce")
    return big_df


def stock_hsgt_board_rank_em(
    symbol: str = "北向资金增持行业板块排行", indicator: str = "今日"
) -> pd.DataFrame:
    """
    东方财富网-数据中心-沪深港通持股-行业板块排行-北向资金增持行业板块排行
    https://data.eastmoney.com/hsgtcg/bk.html
    :param symbol: choice of {"北向资金增持行业板块排行", "北向资金增持概念板块排行", "北向资金增持地域板块排行"}
    :type symbol: str
    :param indicator: choice of {"今日", "3日", "5日", "10日", "1月", "1季", "1年"}
    :type indicator: str
    :return: 北向资金增持行业板块排行
    :rtype: pandas.DataFrame
    """
    url = "https://data.eastmoney.com/hsgtcg/hy.html"
    r = requests.get(url)
    soup = BeautifulSoup(r.text, "lxml")
    current_date = soup.find(attrs={"id": "bkph_date"}).text.strip("（").strip("）")
    symbol_map = {
        "北向资金增持行业板块排行": "5",
        "北向资金增持概念板块排行": "4",
        "北向资金增持地域板块排行": "3",
    }
    indicator_map = {
        "今日": "1",
        "3日": "3",
        "5日": "5",
        "10日": "10",
        "1月": "M",
        "1季": "Q",
        "1年": "Y",
    }
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "ADD_MARKET_CAP",
        "sortTypes": "-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_MUTUAL_BOARD_HOLDRANK_WEB",
        "columns": "ALL",
        "quoteColumns": "f3~05~SECURITY_CODE~INDEX_CHANGE_RATIO",
        "source": "WEB",
        "client": "WEB",
        "filter": f"""(BOARD_TYPE="{symbol_map[symbol]}")(TRADE_DATE='{current_date}')(INTERVAL_TYPE="{indicator_map[indicator]}")""",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = temp_df.index + 1
    temp_df.columns = [
        "序号",
        "_",
        "_",
        "名称",
        "_",
        "最新涨跌幅",
        "报告时间",
        "_",
        "_",
        "北向资金今日增持估计-股票只数",
        "北向资金今日持股-股票只数",
        "北向资金今日增持估计-市值",
        "北向资金今日增持估计-市值增幅",
        "-",
        "北向资金今日增持估计-占板块比",
        "北向资金今日增持估计-占北向资金比",
        "北向资金今日持股-市值",
        "北向资金今日持股-占北向资金比",
        "北向资金今日持股-占板块比",
        "_",
        "_",
        "今日增持最大股-市值",
        "_",
        "_",
        "今日减持最大股-市值",
        "今日增持最大股-占总市值比",
        "_",
        "_",
        "今日减持最大股-占总市值比",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]
    temp_df = temp_df[
        [
            "序号",
            "名称",
            "最新涨跌幅",
            "北向资金今日持股-股票只数",
            "北向资金今日持股-市值",
            "北向资金今日持股-占板块比",
            "北向资金今日持股-占北向资金比",
            "北向资金今日增持估计-股票只数",
            "北向资金今日增持估计-市值",
            "北向资金今日增持估计-市值增幅",
            "北向资金今日增持估计-占板块比",
            "北向资金今日增持估计-占北向资金比",
            "今日增持最大股-市值",
            "今日增持最大股-占总市值比",
            "今日减持最大股-市值",
            "今日减持最大股-占总市值比",
            "报告时间",
        ]
    ]
    temp_df["最新涨跌幅"] = pd.to_numeric(temp_df["最新涨跌幅"], errors="coerce")
    temp_df["北向资金今日持股-股票只数"] = pd.to_numeric(
        temp_df["北向资金今日持股-股票只数"], errors="coerce"
    )
    temp_df["北向资金今日持股-市值"] = pd.to_numeric(
        temp_df["北向资金今日持股-市值"], errors="coerce"
    )
    temp_df["北向资金今日持股-占板块比"] = pd.to_numeric(
        temp_df["北向资金今日持股-占板块比"], errors="coerce"
    )
    temp_df["北向资金今日持股-占北向资金比"] = pd.to_numeric(
        temp_df["北向资金今日持股-占北向资金比"], errors="coerce"
    )
    temp_df["北向资金今日增持估计-股票只数"] = pd.to_numeric(
        temp_df["北向资金今日增持估计-股票只数"], errors="coerce"
    )
    temp_df["北向资金今日增持估计-市值"] = pd.to_numeric(
        temp_df["北向资金今日增持估计-市值"], errors="coerce"
    )
    temp_df["北向资金今日增持估计-市值增幅"] = pd.to_numeric(
        temp_df["北向资金今日增持估计-市值增幅"], errors="coerce"
    )
    temp_df["北向资金今日增持估计-占板块比"] = pd.to_numeric(
        temp_df["北向资金今日增持估计-占板块比"], errors="coerce"
    )
    temp_df["北向资金今日增持估计-占北向资金比"] = pd.to_numeric(
        temp_df["北向资金今日增持估计-占北向资金比"], errors="coerce"
    )
    temp_df["报告时间"] = pd.to_datetime(temp_df["报告时间"], errors="coerce").dt.date
    return temp_df


def stock_hsgt_individual_em(stock: str = "002008") -> pd.DataFrame:
    """
    东方财富-数据中心-沪深港通-沪深港通持股-具体股票
    https://data.eastmoney.com/hsgtcg/StockHdStatistics/002008.html
    :param stock: 股票代码
    :type stock: str
    :return: 具体股票-沪深港通持股
    :rtype: pandas.DataFrame
    """
    url = "http://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "TRADE_DATE",
        "sortTypes": "-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_MUTUAL_HOLDSTOCKNORTH_STA",
        "columns": "ALL",
        "source": "WEB",
        "client": "WEB",
        "filter": f"""(SECURITY_CODE="{stock}")(TRADE_DATE>='2020-07-25')""",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.rename(
        columns={
            "SECURITY_INNER_CODE": "-",
            "SECUCODE": "-",
            "TRADE_DATE": "持股日期",
            "SECURITY_CODE": "-",
            "SECURITY_NAME": "-",
            "MUTUAL_TYPE": "-",
            "CHANGE_RATE": "当日涨跌幅",
            "CLOSE_PRICE": "当日收盘价",
            "HOLD_SHARES": "持股数量",
            "HOLD_MARKET_CAP": "持股市值",
            "A_SHARES_RATIO": "-",
            "HOLD_SHARES_RATIO": "持股数量占A股百分比",
            "FREE_SHARES_RATIO": "-",
            "TOTAL_SHARES_RATIO": "-",
            "HOLD_MARKETCAP_CHG1": "持股市值变化-1日",
            "HOLD_MARKETCAP_CHG5": "持股市值变化-5日",
            "HOLD_MARKETCAP_CHG10": "持股市值变化-10日",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "持股日期",
            "当日收盘价",
            "当日涨跌幅",
            "持股数量",
            "持股市值",
            "持股数量占A股百分比",
            "持股市值变化-1日",
            "持股市值变化-5日",
            "持股市值变化-10日",
        ]
    ]
    temp_df["持股日期"] = pd.to_datetime(temp_df["持股日期"]).dt.date
    temp_df["当日收盘价"] = pd.to_numeric(temp_df["当日收盘价"])
    temp_df["当日涨跌幅"] = pd.to_numeric(temp_df["当日涨跌幅"])
    temp_df["持股数量"] = pd.to_numeric(temp_df["持股数量"])
    temp_df["持股市值"] = pd.to_numeric(temp_df["持股市值"])
    temp_df["持股数量占A股百分比"] = pd.to_numeric(temp_df["持股数量占A股百分比"])
    temp_df["持股市值变化-1日"] = pd.to_numeric(temp_df["持股市值变化-1日"])
    temp_df["持股市值变化-5日"] = pd.to_numeric(temp_df["持股市值变化-5日"])
    temp_df["持股市值变化-10日"] = pd.to_numeric(temp_df["持股市值变化-10日"])
    return temp_df


def stock_hsgt_individual_detail_em(
    symbol: str = "002008",
    start_date: str = "20220130",
    end_date: str = "20220330",
) -> pd.DataFrame:
    """
    东方财富-数据中心-沪深港通-沪深港通持股-具体股票详情
    https://data.eastmoney.com/hsgtcg/StockHdStatistics/002008.html
    :param symbol: 股票代码
    :type symbol: str
    :param start_date: 开始时间
    :type start_date: str
    :param end_date: 结束时间
    :type end_date: str
    :return: 沪深港通持股-具体股票详情
    :rtype: pandas.DataFrame
    """
    url = "http://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "HOLD_DATE",
        "sortTypes": "-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_MUTUAL_HOLD_DET",
        "columns": "ALL",
        "source": "WEB",
        "client": "WEB",
        "filter": f"""(SECURITY_CODE="{symbol}")(MARKET_CODE="003")(HOLD_DATE>='{'-'.join([start_date[:4], start_date[4:6], start_date[6:]])}')(HOLD_DATE<='{'-'.join([end_date[:4], end_date[4:6], end_date[6:]])}')""",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    try:
        data_json["result"]["pages"]
    except TypeError:
        params.update(
            {
                "filter": f"""(SECURITY_CODE="{symbol}")(MARKET_CODE="001")(HOLD_DATE>='{'-'.join([start_date[:4], start_date[4:6], start_date[6:]])}')(HOLD_DATE<='{'-'.join([end_date[:4], end_date[4:6], end_date[6:]])}')""",
            }
        )
        r = requests.get(url, params=params)
        data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    for page in tqdm(range(1, int(total_page) + 1), leave=False):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat([big_df, temp_df], ignore_index=True)
    big_df.rename(
        columns={
            "SECUCODE": "-",
            "SECURITY_CODE": "-",
            "SECURITY_INNER_CODE": "-",
            "SECURITY_NAME_ABBR": "-",
            "HOLD_DATE": "持股日期",
            "ORG_CODE": "-",
            "ORG_NAME": "机构名称",
            "HOLD_NUM": "持股数量",
            "MARKET_CODE": "-",
            "HOLD_SHARES_RATIO": "持股数量占A股百分比",
            "HOLD_MARKET_CAP": "持股市值",
            "CLOSE_PRICE": "当日收盘价",
            "CHANGE_RATE": "当日涨跌幅",
            "HOLD_MARKET_CAPONE": "持股市值变化-1日",
            "HOLD_MARKET_CAPFIVE": "持股市值变化-5日",
            "HOLD_MARKET_CAPTEN": "持股市值变化-10日",
            "PARTICIPANT_CODE": "-",
        },
        inplace=True,
    )
    big_df = big_df[
        [
            "持股日期",
            "当日收盘价",
            "当日涨跌幅",
            "机构名称",
            "持股数量",
            "持股市值",
            "持股数量占A股百分比",
            "持股市值变化-1日",
            "持股市值变化-5日",
            "持股市值变化-10日",
        ]
    ]
    big_df["持股日期"] = pd.to_datetime(big_df["持股日期"]).dt.date
    big_df["当日收盘价"] = pd.to_numeric(big_df["当日收盘价"])
    big_df["当日涨跌幅"] = pd.to_numeric(big_df["当日涨跌幅"])
    big_df["持股数量"] = pd.to_numeric(big_df["持股数量"])
    big_df["持股市值"] = pd.to_numeric(big_df["持股市值"])
    big_df["持股数量占A股百分比"] = pd.to_numeric(big_df["持股数量占A股百分比"])
    big_df["持股市值变化-1日"] = pd.to_numeric(big_df["持股市值变化-1日"])
    big_df["持股市值变化-5日"] = pd.to_numeric(big_df["持股市值变化-5日"])
    big_df["持股市值变化-10日"] = pd.to_numeric(big_df["持股市值变化-10日"])
    return big_df


if __name__ == "__main__":
    stock_hsgt_fund_flow_summary_em_df = stock_hsgt_fund_flow_summary_em()
    print(stock_hsgt_fund_flow_summary_em_df)

    stock_hk_ggt_components_em_df = stock_hk_ggt_components_em()
    print(stock_hk_ggt_components_em_df)

    stock_hsgt_hold_stock_em_df = stock_hsgt_hold_stock_em(
        market="北向", indicator="今日排行"
    )
    print(stock_hsgt_hold_stock_em_df)

    stock_hsgt_hold_stock_em_df = stock_hsgt_hold_stock_em(
        market="沪股通", indicator="5日排行"
    )
    print(stock_hsgt_hold_stock_em_df)

    stock_hsgt_hold_stock_em_df = stock_hsgt_hold_stock_em(
        market="深股通", indicator="5日排行"
    )
    print(stock_hsgt_hold_stock_em_df)

    stock_hsgt_hold_stock_em_df = stock_hsgt_hold_stock_em(
        market="沪股通", indicator="10日排行"
    )
    print(stock_hsgt_hold_stock_em_df)

    stock_hsgt_stock_statistics_em_df = stock_hsgt_stock_statistics_em(
        symbol="北向持股", start_date="20240110", end_date="20240110"
    )
    print(stock_hsgt_stock_statistics_em_df)

    stock_hsgt_stock_statistics_em_df = stock_hsgt_stock_statistics_em(
        symbol="南向持股", start_date="20240110", end_date="20240110"
    )
    print(stock_hsgt_stock_statistics_em_df)

    stock_hsgt_stock_statistics_em_df = stock_hsgt_stock_statistics_em(
        symbol="沪股通持股", start_date="20240110", end_date="20240110"
    )
    print(stock_hsgt_stock_statistics_em_df)

    stock_hsgt_stock_statistics_em_df = stock_hsgt_stock_statistics_em(
        symbol="深股通持股", start_date="20240110", end_date="20240110"
    )
    print(stock_hsgt_stock_statistics_em_df)

    stock_hsgt_institution_statistics_em_df = stock_hsgt_institution_statistics_em(
        market="北向持股", start_date="20240110", end_date="20240110"
    )
    print(stock_hsgt_institution_statistics_em_df)

    stock_hsgt_institution_statistics_em_df = stock_hsgt_institution_statistics_em(
        market="南向持股", start_date="20240110", end_date="20240110"
    )
    print(stock_hsgt_institution_statistics_em_df)

    stock_hsgt_institution_statistics_em_df = stock_hsgt_institution_statistics_em(
        market="沪股通持股", start_date="20240110", end_date="20240110"
    )
    print(stock_hsgt_institution_statistics_em_df)

    stock_hsgt_institution_statistics_em_df = stock_hsgt_institution_statistics_em(
        market="深股通持股", start_date="20240110", end_date="20240110"
    )
    print(stock_hsgt_institution_statistics_em_df)

    stock_hsgt_hist_em_df = stock_hsgt_hist_em(symbol="北向资金")
    print(stock_hsgt_hist_em_df)

    stock_hsgt_board_rank_em_df = stock_hsgt_board_rank_em(
        symbol="北向资金增持行业板块排行", indicator="今日"
    )
    print(stock_hsgt_board_rank_em_df)

    stock_hsgt_individual_em_df = stock_hsgt_individual_em(stock="002008")
    print(stock_hsgt_individual_em_df)

    stock_hsgt_individual_detail_em_df = stock_hsgt_individual_detail_em(
        symbol="002008", start_date="20240110", end_date="20240110"
    )
    print(stock_hsgt_individual_detail_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/5/10 17:06
Desc: 提供英为财情-国际大宗商品期货
https://cn.investing.com/commodities/brent-oil-historical-data
"""
import re

import pandas as pd
import requests
from bs4 import BeautifulSoup

from byteapp_ak.index.cons import short_headers, long_headers


def get_sector_symbol_name_url() -> dict:
    """
    期货所对应板块的 URL
    :return: dict
    {'能源': '/commodities/energy',
    '金属': '/commodities/metals',
    '农业': '/commodities/softs',
    '商品指数': '/indices/commodities-indices'}
    """
    url = "https://cn.investing.com/commodities/"
    res = requests.get(url, headers=short_headers)
    soup = BeautifulSoup(res.text, "lxml")
    name_url_option_list = soup.find_all(attrs={"class": "linkTitle"})  # 去掉-所有国家及地区
    url_list = [item.find("a")["href"] for item in name_url_option_list]
    name_list = [item.get_text() for item in name_url_option_list]
    name_code_map_dict = {}
    name_code_map_dict.update(zip(name_list, url_list))
    return name_code_map_dict


def futures_global_commodity_name_url_map(sector: str = "能源") -> dict:
    """
    参考网页: https://cn.investing.com/commodities/
    获取选择板块对应的: 具体期货品种的 url 地址
    :param sector: 板块, 对应 get_global_country_name_url 品种名称
    :type sector: str
    :return: dict of name-url
    :rtype: dict
    {'伦敦布伦特原油': '/commodities/brent-oil',
    'WTI原油': '/commodities/crude-oil',
    '伦敦汽油': '/commodities/london-gas-oil',
    '天然气': '/commodities/natural-gas?cid=49787',
    '燃料油': '/commodities/heating-oil',
    '碳排放': '/commodities/carbon-emissions',
    'RBOB汽油': '/commodities/gasoline-rbob',
    '布伦特原油': '/commodities/brent-oil?cid=49769',
    '原油': '/commodities/crude-oil?cid=49774'}
    """
    name_url_dict = get_sector_symbol_name_url()
    url = f"https://cn.investing.com{name_url_dict[sector]}"
    res = requests.post(url, headers=short_headers)
    soup = BeautifulSoup(res.text, "lxml")
    url_list = [
        item.find("a")["href"].split("?")[0]
        for item in soup.find_all(attrs={"class": "plusIconTd"})
    ]
    name_list = [
        item.find("a").get_text()
        for item in soup.find_all(attrs={"class": "plusIconTd"})
    ]
    name_code_map_dict = {}
    name_code_map_dict.update(zip(name_list, url_list))
    return name_code_map_dict


def futures_global_commodity_hist(
    sector: str = "能源",
    symbol: str = "伦敦布伦特原油",
    start_date: str = "20000101",
    end_date: str = "20191017",
) -> pd.DataFrame:
    """
    国际大宗商品的历史量价数据
    https://cn.investing.com/commodities
    :param sector: 板块名称; 调用 futures_global_commodity_name_url_map 函数获取
    :type sector: str
    :param symbol: 品种名称; 通过访问网站查询
    :type symbol: str
    :param start_date: 开始日期
    :type start_date: str
    :param end_date: 结束日期
    :type end_date: str
    :return: 国际大宗商品的历史量价数据
    :rtype: pandas.DataFrame
    """
    start_date = "/".join([start_date[:4], start_date[4:6], start_date[6:]])
    end_date = "/".join([end_date[:4], end_date[4:6], end_date[6:]])
    name_code_dict = futures_global_commodity_name_url_map(sector)
    temp_url = f"https://cn.investing.com/{name_code_dict[symbol]}-historical-data"
    res = requests.post(temp_url, headers=short_headers)
    soup = BeautifulSoup(res.text, "lxml")
    title = soup.find("h2", attrs={"class": "float_lang_base_1"}).get_text()
    res = requests.post(temp_url, headers=short_headers)
    soup = BeautifulSoup(res.text, "lxml")
    data = soup.find_all(text=re.compile("window.histDataExcessInfo"))[0].strip()
    para_data = re.findall(r"\d+", data)
    payload = {
        "curr_id": para_data[0],
        "smlID": para_data[1],
        "header": title,
        "st_date": start_date,
        "end_date": end_date,
        "interval_sec": "Daily",
        "sort_col": "date",
        "sort_ord": "DESC",
        "action": "historical_data",
    }
    url = "https://cn.investing.com/instruments/HistoricalDataAjax"
    r = requests.post(url, data=payload, headers=long_headers)
    temp_df = pd.read_html(r.text)[0]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], format="%Y年%m月%d日")
    if any(temp_df["交易量"].astype(str).str.contains("-")):
        temp_df["交易量"][temp_df["交易量"].str.contains("-")] = temp_df["交易量"][
            temp_df["交易量"].str.contains("-")
        ].replace("-", 0)
    if any(temp_df["交易量"].astype(str).str.contains("B")):
        temp_df["交易量"][temp_df["交易量"].str.contains("B").fillna(False)] = (
            temp_df["交易量"][temp_df["交易量"].str.contains("B").fillna(False)]
            .str.replace("B", "")
            .astype(float)
            * 1000000000
        )
    if any(temp_df["交易量"].astype(str).str.contains("M")):
        temp_df["交易量"][temp_df["交易量"].str.contains("M").fillna(False)] = (
            temp_df["交易量"][temp_df["交易量"].str.contains("M").fillna(False)]
            .str.replace("M", "")
            .astype(float)
            * 1000000
        )
    if any(temp_df["交易量"].astype(str).str.contains("K")):
        temp_df["交易量"][temp_df["交易量"].str.contains("K").fillna(False)] = (
            temp_df["交易量"][temp_df["交易量"].str.contains("K").fillna(False)]
            .str.replace("K", "")
            .astype(float)
            * 1000
        )
    temp_df["交易量"] = temp_df["交易量"].astype(float)
    temp_df["涨跌幅"] = pd.DataFrame(
        round(temp_df["涨跌幅"].str.replace("%", "").astype(float) / 100, 6)
    )
    temp_df.name = title
    temp_df.columns.name = None
    temp_df.sort_values(["日期"], ascending=False, inplace=True)
    return temp_df


if __name__ == "__main__":
    temp_dict = futures_global_commodity_name_url_map(sector="能源")
    print(temp_dict)

    futures_global_commodity_hist_df = futures_global_commodity_hist(
        sector="能源", symbol="伦敦布伦特原油", start_date="19700101", end_date="20210510"
    )
    print(futures_global_commodity_hist_df)

    # futures_global_commodity_hist_df = futures_global_commodity_hist(
    #     sector="能源", symbol="伦敦布伦特原油", start_date="1970/01/01", end_date="2021/05/10"
    # )
    # print(futures_global_commodity_hist_df.to_csv("伦敦布伦特原油_19880627_20080319.csv", encoding="gb2312"))
    #
    # futures_global_commodity_hist_df = futures_global_commodity_hist(
    #     sector="能源", symbol="伦敦布伦特原油", start_date="2008/03/19", end_date="2021/05/10"
    # )
    # print(futures_global_commodity_hist_df.to_csv("伦敦布伦特原油_20080319_20210510.csv", encoding="gb2312"))



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2022/1/7 13:40
Desc: 新浪财经-机构推荐池
http://stock.finance.sina.com.cn/stock/go.php/vIR_RatingNewest/index.phtml
"""
import pandas as pd
import requests
from bs4 import BeautifulSoup


def stock_institute_recommend(symbol: str = "投资评级选股") -> pd.DataFrame:
    """
    新浪财经-机构推荐池-最新投资评级
    http://stock.finance.sina.com.cn/stock/go.php/vIR_RatingNewest/index.phtml
    :param symbol: choice of {'最新投资评级', '上调评级股票', '下调评级股票', '股票综合评级', '首次评级股票', '目标涨幅排名', '机构关注度', '行业关注度', '投资评级选股'}
    :type symbol: str
    :return: 最新投资评级数据
    :rtype: pandas.DataFrame
    """
    url = "http://stock.finance.sina.com.cn/stock/go.php/vIR_RatingNewest/index.phtml"
    params = {
        "num": "40",
        "p": "1",
    }
    r = requests.get(url, params=params)
    soup = BeautifulSoup(r.text, "lxml")
    indicator_map = {item.find("a").text: item.find("a")["href"] for item in soup.find(attrs={"id": "leftMenu"}).find_all("dd")[1].find_all("li")}
    url = indicator_map[symbol]
    params = {
        "num": "10000",
        "p": "1",
    }
    r = requests.get(url, params=params)
    if symbol == "股票综合评级":
        temp_df = pd.read_html(r.text, header=0)[0].iloc[:, :9]
        temp_df["股票代码"] = temp_df["股票代码"].astype(str).str.zfill(6)
        temp_df = temp_df.rename(columns={"综合评级↓": "综合评级"})
        return temp_df
    if symbol == "首次评级股票":
        temp_df = pd.read_html(r.text, header=0)[0].iloc[:, :8]
        temp_df["股票代码"] = temp_df["股票代码"].astype(str).str.zfill(6)
        temp_df = temp_df.rename(columns={"评级日期↓": "评级日期"})
        return temp_df
    if symbol == "目标涨幅排名":
        temp_df = pd.read_html(r.text, header=0)[0].iloc[:, :7]
        temp_df["股票代码"] = temp_df["股票代码"].astype(str).str.zfill(6)
        temp_df = temp_df.rename(columns={"平均目标涨幅↓": "平均目标涨幅"})
        return temp_df
    if symbol == "机构关注度":
        temp_df = pd.read_html(r.text, header=0)[0].iloc[:, :11]
        temp_df["股票代码"] = temp_df["股票代码"].astype(str).str.zfill(6)
        temp_df = temp_df.rename(columns={"关注度↓": "关注度"})
        return temp_df
    if symbol == "行业关注度":
        temp_df = pd.read_html(r.text, header=0)[0].iloc[:, :11]
        temp_df = temp_df.rename(columns={"关注度↓": "关注度"})
        return temp_df
    if symbol == "投资评级选股":
        temp_df = pd.read_html(r.text, header=0)[0].iloc[:, :9]
        temp_df["股票代码"] = temp_df["股票代码"].astype(str).str.zfill(6)
        del temp_df["评级明细"]
        temp_df = temp_df.rename(columns={"评级日期↓": "评级日期"})
        return temp_df
    temp_df = pd.read_html(r.text, header=0)[0].iloc[:, :8]
    temp_df["股票代码"] = temp_df["股票代码"].astype(str).str.zfill(6)
    temp_df = temp_df.rename(columns={"评级日期↓": "评级日期"})
    return temp_df


def stock_institute_recommend_detail(symbol: str = "000001") -> pd.DataFrame:
    """
    新浪财经-机构推荐池-股票评级记录
    http://stock.finance.sina.com.cn/stock/go.php/vIR_StockSearch/key/sz000001.phtml
    :param symbol: 股票代码
    :type symbol: str
    :return: 具体股票的股票评级记录
    :rtype: pandas.DataFrame
    """
    url = f"http://stock.finance.sina.com.cn/stock/go.php/vIR_StockSearch/key/{symbol}.phtml"
    params = {
        "num": "5000",
        "p": "1",
    }
    r = requests.get(url, params=params)
    temp_df = pd.read_html(r.text, header=0)[0].iloc[:, :8]
    temp_df["股票代码"] = temp_df["股票代码"].astype(str).str.zfill(6)
    temp_df = temp_df.rename(columns={"评级日期↓": "评级日期"})
    return temp_df


if __name__ == '__main__':
    for item in ['最新投资评级', '上调评级股票', '下调评级股票', '股票综合评级', '首次评级股票', '目标涨幅排名', '机构关注度', '行业关注度', '投资评级选股']:
        stock_institute_recommend_df = stock_institute_recommend(symbol=item)
        print(stock_institute_recommend_df)

    stock_institute_recommend_detail_df = stock_institute_recommend_detail(symbol="002709")
    print(stock_institute_recommend_detail_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2022/11/27 20:30
Desc: 东方财富-经济数据-加拿大
https://data.eastmoney.com/cjsj/foreign_5_0.html
"""
import pandas as pd
import requests


# 新屋开工
def macro_canada_new_house_rate() -> pd.DataFrame:
    """
    东方财富-经济数据-加拿大-新屋开工
    https://data.eastmoney.com/cjsj/foreign_7_0.html
    :return: 新屋开工
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_ECONOMICVALUE_CA",
        "columns": "ALL",
        "filter": '(INDICATOR_ID="EMG00342247")',
        "pageNumber": "1",
        "pageSize": "2000",
        "sortColumns": "REPORT_DATE",
        "sortTypes": "-1",
        "source": "WEB",
        "client": "WEB",
        "p": "1",
        "pageNo": "1",
        "pageNum": "1",
        "_": "1669047266881",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.columns = [
        "-",
        "-",
        "-",
        "-",
        "时间",
        "-",
        "发布日期",
        "现值",
        "前值",
    ]
    temp_df = temp_df[[
        "时间",
        "前值",
        "现值",
        "发布日期",
    ]]
    temp_df["前值"] = pd.to_numeric(temp_df["前值"], errors="coerce")
    temp_df["现值"] = pd.to_numeric(temp_df["现值"], errors="coerce")
    temp_df['发布日期'] = pd.to_datetime(temp_df['发布日期']).dt.date
    return temp_df


# 失业率
def macro_canada_unemployment_rate() -> pd.DataFrame:
    """
    东方财富-经济数据-加拿大-失业率
    https://data.eastmoney.com/cjsj/foreign_7_1.html
    :return: 失业率
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_ECONOMICVALUE_CA",
        "columns": "ALL",
        "filter": '(INDICATOR_ID="EMG00157746")',
        "pageNumber": "1",
        "pageSize": "2000",
        "sortColumns": "REPORT_DATE",
        "sortTypes": "-1",
        "source": "WEB",
        "client": "WEB",
        "p": "1",
        "pageNo": "1",
        "pageNum": "1",
        "_": "1669047266881",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.columns = [
        "-",
        "-",
        "-",
        "-",
        "时间",
        "-",
        "发布日期",
        "现值",
        "前值",
    ]
    temp_df = temp_df[[
        "时间",
        "前值",
        "现值",
        "发布日期",
    ]]
    temp_df["前值"] = pd.to_numeric(temp_df["前值"], errors="coerce")
    temp_df["现值"] = pd.to_numeric(temp_df["现值"], errors="coerce")
    temp_df['发布日期'] = pd.to_datetime(temp_df['发布日期']).dt.date
    return temp_df


# 贸易帐
def macro_canada_trade() -> pd.DataFrame:
    """
    东方财富-经济数据-加拿大-贸易帐
    https://data.eastmoney.com/cjsj/foreign_7_2.html
    :return: 贸易帐
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_ECONOMICVALUE_CA",
        "columns": "ALL",
        "filter": '(INDICATOR_ID="EMG00102022")',
        "pageNumber": "1",
        "pageSize": "2000",
        "sortColumns": "REPORT_DATE",
        "sortTypes": "-1",
        "source": "WEB",
        "client": "WEB",
        "p": "1",
        "pageNo": "1",
        "pageNum": "1",
        "_": "1669047266881",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.columns = [
        "-",
        "-",
        "-",
        "-",
        "时间",
        "-",
        "发布日期",
        "现值",
        "前值",
    ]
    temp_df = temp_df[[
        "时间",
        "前值",
        "现值",
        "发布日期",
    ]]
    temp_df["前值"] = pd.to_numeric(temp_df["前值"], errors="coerce")
    temp_df["现值"] = pd.to_numeric(temp_df["现值"], errors="coerce")
    temp_df['发布日期'] = pd.to_datetime(temp_df['发布日期']).dt.date
    return temp_df


# 零售销售月率
def macro_canada_retail_rate_monthly() -> pd.DataFrame:
    """
    东方财富-经济数据-加拿大-零售销售月率
    https://data.eastmoney.com/cjsj/foreign_7_3.html
    :return: 零售销售月率
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_ECONOMICVALUE_CA",
        "columns": "ALL",
        "filter": '(INDICATOR_ID="EMG01337094")',
        "pageNumber": "1",
        "pageSize": "2000",
        "sortColumns": "REPORT_DATE",
        "sortTypes": "-1",
        "source": "WEB",
        "client": "WEB",
        "p": "1",
        "pageNo": "1",
        "pageNum": "1",
        "_": "1669047266881",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.columns = [
        "-",
        "-",
        "-",
        "-",
        "时间",
        "-",
        "发布日期",
        "现值",
        "前值",
    ]
    temp_df = temp_df[[
        "时间",
        "前值",
        "现值",
        "发布日期",
    ]]
    temp_df["前值"] = pd.to_numeric(temp_df["前值"], errors="coerce")
    temp_df["现值"] = pd.to_numeric(temp_df["现值"], errors="coerce")
    temp_df['发布日期'] = pd.to_datetime(temp_df['发布日期']).dt.date
    return temp_df


# 央行公布利率决议
def macro_canada_bank_rate() -> pd.DataFrame:
    """
    东方财富-经济数据-加拿大-央行公布利率决议
    https://data.eastmoney.com/cjsj/foreign_7_4.html
    :return: 央行公布利率决议
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_ECONOMICVALUE_CA",
        "columns": "ALL",
        "filter": '(INDICATOR_ID="EMG00342248")',
        "pageNumber": "1",
        "pageSize": "2000",
        "sortColumns": "REPORT_DATE",
        "sortTypes": "-1",
        "source": "WEB",
        "client": "WEB",
        "p": "1",
        "pageNo": "1",
        "pageNum": "1",
        "_": "1669047266881",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.columns = [
        "-",
        "-",
        "-",
        "-",
        "时间",
        "-",
        "发布日期",
        "现值",
        "前值",
    ]
    temp_df = temp_df[[
        "时间",
        "前值",
        "现值",
        "发布日期",
    ]]
    temp_df["前值"] = pd.to_numeric(temp_df["前值"], errors="coerce")
    temp_df["现值"] = pd.to_numeric(temp_df["现值"], errors="coerce")
    temp_df['发布日期'] = pd.to_datetime(temp_df['发布日期']).dt.date
    return temp_df


# 核心消费者物价指数年率
def macro_canada_core_cpi_yearly() -> pd.DataFrame:
    """
    东方财富-经济数据-加拿大-核心消费者物价指数年率
    https://data.eastmoney.com/cjsj/foreign_7_5.html
    :return: 核心消费者物价指数年率
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_ECONOMICVALUE_CA",
        "columns": "ALL",
        "filter": '(INDICATOR_ID="EMG00102030")',
        "pageNumber": "1",
        "pageSize": "2000",
        "sortColumns": "REPORT_DATE",
        "sortTypes": "-1",
        "source": "WEB",
        "client": "WEB",
        "p": "1",
        "pageNo": "1",
        "pageNum": "1",
        "_": "1669047266881",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.columns = [
        "-",
        "-",
        "-",
        "-",
        "时间",
        "-",
        "发布日期",
        "现值",
        "前值",
    ]
    temp_df = temp_df[[
        "时间",
        "前值",
        "现值",
        "发布日期",
    ]]
    temp_df["前值"] = pd.to_numeric(temp_df["前值"], errors="coerce")
    temp_df["现值"] = pd.to_numeric(temp_df["现值"], errors="coerce")
    temp_df['发布日期'] = pd.to_datetime(temp_df['发布日期']).dt.date
    return temp_df


# 核心消费者物价指数月率
def macro_canada_core_cpi_monthly() -> pd.DataFrame:
    """
    东方财富-经济数据-加拿大-核心消费者物价指数月率
    https://data.eastmoney.com/cjsj/foreign_7_6.html
    :return: 核心消费者物价指数月率
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_ECONOMICVALUE_CA",
        "columns": "ALL",
        "filter": '(INDICATOR_ID="EMG00102044")',
        "pageNumber": "1",
        "pageSize": "2000",
        "sortColumns": "REPORT_DATE",
        "sortTypes": "-1",
        "source": "WEB",
        "client": "WEB",
        "p": "1",
        "pageNo": "1",
        "pageNum": "1",
        "_": "1669047266881",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.columns = [
        "-",
        "-",
        "-",
        "-",
        "时间",
        "-",
        "发布日期",
        "现值",
        "前值",
    ]
    temp_df = temp_df[[
        "时间",
        "前值",
        "现值",
        "发布日期",
    ]]
    temp_df["前值"] = pd.to_numeric(temp_df["前值"], errors="coerce")
    temp_df["现值"] = pd.to_numeric(temp_df["现值"], errors="coerce")
    temp_df['发布日期'] = pd.to_datetime(temp_df['发布日期']).dt.date
    return temp_df


# 消费者物价指数年率
def macro_canada_cpi_yearly() -> pd.DataFrame:
    """
    东方财富-经济数据-加拿大-消费者物价指数年率
    https://data.eastmoney.com/cjsj/foreign_7_7.html
    :return: 消费者物价指数年率
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_ECONOMICVALUE_CA",
        "columns": "ALL",
        "filter": '(INDICATOR_ID="EMG00102029")',
        "pageNumber": "1",
        "pageSize": "2000",
        "sortColumns": "REPORT_DATE",
        "sortTypes": "-1",
        "source": "WEB",
        "client": "WEB",
        "p": "1",
        "pageNo": "1",
        "pageNum": "1",
        "_": "1669047266881",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.columns = [
        "-",
        "-",
        "-",
        "-",
        "时间",
        "-",
        "发布日期",
        "现值",
        "前值",
    ]
    temp_df = temp_df[[
        "时间",
        "前值",
        "现值",
        "发布日期",
    ]]
    temp_df["前值"] = pd.to_numeric(temp_df["前值"], errors="coerce")
    temp_df["现值"] = pd.to_numeric(temp_df["现值"], errors="coerce")
    temp_df['发布日期'] = pd.to_datetime(temp_df['发布日期']).dt.date
    return temp_df


# 消费者物价指数月率
def macro_canada_cpi_monthly() -> pd.DataFrame:
    """
    东方财富-经济数据-加拿大-消费者物价指数月率
    https://data.eastmoney.com/cjsj/foreign_7_8.html
    :return: 消费者物价指数月率
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_ECONOMICVALUE_CA",
        "columns": "ALL",
        "filter": '(INDICATOR_ID="EMG00158719")',
        "pageNumber": "1",
        "pageSize": "2000",
        "sortColumns": "REPORT_DATE",
        "sortTypes": "-1",
        "source": "WEB",
        "client": "WEB",
        "p": "1",
        "pageNo": "1",
        "pageNum": "1",
        "_": "1669047266881",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.columns = [
        "-",
        "-",
        "-",
        "-",
        "时间",
        "-",
        "发布日期",
        "现值",
        "前值",
    ]
    temp_df = temp_df[[
        "时间",
        "前值",
        "现值",
        "发布日期",
    ]]
    temp_df["前值"] = pd.to_numeric(temp_df["前值"], errors="coerce")
    temp_df["现值"] = pd.to_numeric(temp_df["现值"], errors="coerce")
    temp_df['发布日期'] = pd.to_datetime(temp_df['发布日期']).dt.date
    return temp_df


# GDP 月率
def macro_canada_gdp_monthly() -> pd.DataFrame:
    """
    东方财富-经济数据-加拿大-GDP 月率
    https://data.eastmoney.com/cjsj/foreign_7_9.html
    :return: GDP 月率
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_ECONOMICVALUE_CA",
        "columns": "ALL",
        "filter": '(INDICATOR_ID="EMG00159259")',
        "pageNumber": "1",
        "pageSize": "2000",
        "sortColumns": "REPORT_DATE",
        "sortTypes": "-1",
        "source": "WEB",
        "client": "WEB",
        "p": "1",
        "pageNo": "1",
        "pageNum": "1",
        "_": "1669047266881",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.columns = [
        "-",
        "-",
        "-",
        "-",
        "时间",
        "-",
        "发布日期",
        "现值",
        "前值",
    ]
    temp_df = temp_df[[
        "时间",
        "前值",
        "现值",
        "发布日期",
    ]]
    temp_df["前值"] = pd.to_numeric(temp_df["前值"], errors="coerce")
    temp_df["现值"] = pd.to_numeric(temp_df["现值"], errors="coerce")
    temp_df['发布日期'] = pd.to_datetime(temp_df['发布日期']).dt.date
    return temp_df


if __name__ == '__main__':
    macro_canada_new_house_rate_df = macro_canada_new_house_rate()
    print(macro_canada_new_house_rate_df)

    macro_canada_unemployment_rate_df = macro_canada_unemployment_rate()
    print(macro_canada_unemployment_rate_df)

    macro_canada_trade_df = macro_canada_trade()
    print(macro_canada_trade_df)

    macro_canada_retail_rate_monthly_df = macro_canada_retail_rate_monthly()
    print(macro_canada_retail_rate_monthly_df)

    macro_canada_bank_rate_df = macro_canada_bank_rate()
    print(macro_canada_bank_rate_df)

    macro_canada_core_cpi_yearly_df = macro_canada_core_cpi_yearly()
    print(macro_canada_core_cpi_yearly_df)

    macro_canada_core_cpi_monthly_df = macro_canada_core_cpi_monthly()
    print(macro_canada_core_cpi_monthly_df)

    macro_canada_cpi_yearly_df = macro_canada_cpi_yearly()
    print(macro_canada_cpi_yearly_df)

    macro_canada_cpi_monthly_df = macro_canada_cpi_monthly()
    print(macro_canada_cpi_monthly_df)

    macro_canada_gdp_monthly_df = macro_canada_gdp_monthly()
    print(macro_canada_gdp_monthly_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/4/17 19:17
Desc: 义乌小商品指数
https://www.ywindex.com/Home/Product/index/
"""

import pandas as pd
import requests
from bs4 import BeautifulSoup


def index_yw(symbol: str = "月景气指数") -> pd.DataFrame:
    """
    义乌小商品指数
    https://www.ywindex.com/Home/Product/index/
    :param symbol: choice of {"周价格指数", "月价格指数", "月景气指数"}
    :type symbol: str
    :return: 指数结果
    :rtype: pandas.DataFrame
    """
    import urllib3

    # 禁用InsecureRequestWarning
    urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
    name_num_dict = {
        "周价格指数": 1,
        "月价格指数": 3,
        "月景气指数": 5,
    }
    url = "https://www.ywindex.com/Home/Product/index/"
    res = requests.get(url, verify=False)
    soup = BeautifulSoup(res.text, features="lxml")
    table_name = (
        soup.find_all(attrs={"class": "tablex"})[name_num_dict[symbol]]
        .get_text()
        .split("\n\n\n\n\n")[2]
        .split("\n")
    )
    table_content = (
        soup.find_all(attrs={"class": "tablex"})[name_num_dict[symbol]]
        .get_text()
        .split("\n\n\n\n\n")[3]
        .split("\n\n")
    )
    if symbol == "月景气指数":
        table_df = pd.DataFrame([item.split("\n") for item in table_content]).iloc[
            :, :5
        ]
        table_df.columns = ["期数", "景气指数", "规模指数", "效益指数", "市场信心指数"]
        table_df["期数"] = pd.to_datetime(table_df["期数"], errors="coerce").dt.date
        table_df["景气指数"] = pd.to_numeric(table_df["景气指数"], errors="coerce")
        table_df["规模指数"] = pd.to_numeric(table_df["规模指数"], errors="coerce")
        table_df["效益指数"] = pd.to_numeric(table_df["效益指数"], errors="coerce")
        table_df["市场信心指数"] = pd.to_numeric(
            table_df["市场信心指数"], errors="coerce"
        )
        table_df.sort_values(["期数"], inplace=True, ignore_index=True)
        return table_df
    elif symbol == "周价格指数":
        table_df = pd.DataFrame([item.split("\n") for item in table_content]).iloc[
            :, :6
        ]
        table_df.columns = table_name
        table_df["期数"] = pd.to_datetime(table_df["期数"], errors="coerce").dt.date
        table_df["价格指数"] = pd.to_numeric(table_df["价格指数"], errors="coerce")
        table_df["场内价格指数"] = pd.to_numeric(
            table_df["场内价格指数"], errors="coerce"
        )
        table_df["网上价格指数"] = pd.to_numeric(
            table_df["网上价格指数"], errors="coerce"
        )
        table_df["订单价格指数"] = pd.to_numeric(
            table_df["订单价格指数"], errors="coerce"
        )
        table_df["出口价格指数"] = pd.to_numeric(
            table_df["出口价格指数"], errors="coerce"
        )
        table_df.sort_values(["期数"], inplace=True, ignore_index=True)
        return table_df
    elif symbol == "月价格指数":
        table_df = pd.DataFrame([item.split("\n") for item in table_content]).iloc[
            :, :6
        ]
        table_df.columns = table_name
        table_df["期数"] = pd.to_datetime(table_df["期数"], errors="coerce").dt.date
        table_df["价格指数"] = pd.to_numeric(table_df["价格指数"], errors="coerce")
        table_df["场内价格指数"] = pd.to_numeric(
            table_df["场内价格指数"], errors="coerce"
        )
        table_df["网上价格指数"] = pd.to_numeric(
            table_df["网上价格指数"], errors="coerce"
        )
        table_df["订单价格指数"] = pd.to_numeric(
            table_df["订单价格指数"], errors="coerce"
        )
        table_df["出口价格指数"] = pd.to_numeric(
            table_df["出口价格指数"], errors="coerce"
        )
        table_df.sort_values(["期数"], inplace=True, ignore_index=True)
        return table_df


if __name__ == "__main__":
    index_yw_df = index_yw(symbol="周价格指数")
    print(index_yw_df)

    index_yw_df = index_yw(symbol="月价格指数")
    print(index_yw_df)

    index_yw_df = index_yw(symbol="月景气指数")
    print(index_yw_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/4/7 15:05
Desc: 乐咕乐股-大盘拥挤度
https://legulegu.com/stockdata/ashares-congestion
"""
import pandas as pd
import requests

from byteapp_ak.stock_feature.stock_a_indicator import get_token_lg, get_cookie_csrf


def stock_a_congestion_lg() -> pd.DataFrame:
    """
    乐咕乐股-大盘拥挤度
    https://legulegu.com/stockdata/ashares-congestion
    :return: 大盘拥挤度
    :rtype: pandas.DataFrame
    """
    url = "https://legulegu.com/api/stockdata/ashares-congestion"
    token = get_token_lg()
    params = {"token": token}
    r = requests.get(
        url,
        params=params,
        **get_cookie_csrf(url="https://legulegu.com/stockdata/ashares-congestion")
    )
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["items"])
    temp_df["date"] = pd.to_datetime(temp_df["date"]).dt.date
    temp_df = temp_df[
        [
            "date",
            "close",
            "congestion",
        ]
    ]
    temp_df["close"] = pd.to_numeric(temp_df["close"], errors="coerce")
    temp_df["congestion"] = pd.to_numeric(temp_df["congestion"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    stock_a_congestion_lg_df = stock_a_congestion_lg()
    print(stock_a_congestion_lg_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2024/7/24 23:30
Desc: 巨潮资讯-数据中心-专题统计-基金报表
https://webapi.cninfo.com.cn/#/thematicStatistics
"""

import pandas as pd
import py_mini_racer
import requests

from byteapp_ak.datasets import get_ths_js


def _get_file_content_cninfo(file: str = "cninfo.js") -> str:
    """
    获取 JS 文件的内容
    :param file:  JS 文件名
    :type file: str
    :return: 文件内容
    :rtype: str
    """
    setting_file_path = get_ths_js(file)
    with open(setting_file_path, encoding="utf-8") as f:
        file_data = f.read()
    return file_data


def fund_report_stock_cninfo(date: str = "20210630") -> pd.DataFrame:
    """
    巨潮资讯-数据中心-专题统计-基金报表-基金重仓股
    https://webapi.cninfo.com.cn/#/thematicStatistics
    :param date: 报告时间; choice of {"XXXX0331", "XXXX0630", "XXXX0930", "XXXX1231"}
    :type date: str
    :return: 基金重仓股
    :rtype: pandas.DataFrame
    """
    url = "https://webapi.cninfo.com.cn/api/sysapi/p_sysapi1112"
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_cninfo("cninfo.js")
    js_code.eval(js_content)
    mcode = js_code.call("getResCode1")
    headers = {
        "Accept": "*/*",
        "Accept-Enckey": mcode,
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "Origin": "https://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "https://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    params = {
        "rdate": "-".join([date[:4], date[4:6], date[6:]]),
    }
    r = requests.post(url, headers=headers, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.rename(
        columns={
            "F003N": "持股总市值",
            "F002N": "持股总数",
            "F001N": "基金覆盖家数",
            "SECNAME": "股票简称",
            "ID": "序号",
            "SECCODE": "股票代码",
            "ENDDATE": "报告期",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "序号",
            "股票代码",
            "股票简称",
            "报告期",
            "基金覆盖家数",
            "持股总数",
            "持股总市值",
        ]
    ]
    temp_df["报告期"] = pd.to_datetime(temp_df["报告期"], errors="coerce").dt.date
    temp_df["持股总数"] = pd.to_numeric(temp_df["持股总数"], errors="coerce")
    temp_df["持股总市值"] = pd.to_numeric(temp_df["持股总市值"], errors="coerce")
    temp_df["基金覆盖家数"] = pd.to_numeric(temp_df["基金覆盖家数"], errors="coerce")
    temp_df["序号"] = range(1, len(temp_df) + 1)
    return temp_df


def fund_report_industry_allocation_cninfo(date: str = "20210630") -> pd.DataFrame:
    """
    巨潮资讯-数据中心-专题统计-基金报表-基金行业配置
    https://webapi.cninfo.com.cn/#/thematicStatistics
    :param date: 报告时间; choice of {"XXXX0331", "XXXX0630", "XXXX0930", "XXXX1231"}, 从 2017 年开始
    :type date: str
    :return: 基金行业配置
    :rtype: pandas.DataFrame
    """
    url = "https://webapi.cninfo.com.cn/api/sysapi/p_sysapi1113"
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_cninfo("cninfo.js")
    js_code.eval(js_content)
    mcode = js_code.call("getResCode1")
    headers = {
        "Accept": "*/*",
        "Accept-Enckey": mcode,
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "Origin": "https://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "https://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    params = {
        "rdate": "-".join([date[:4], date[4:6], date[6:]]),
    }
    r = requests.post(url, headers=headers, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.rename(
        columns={
            "F004N": "行业规模",
            "F003N": "基金覆盖家数",
            "F002V": "证监会行业名称",
            "F001V": "行业编码",
            "ENDDATE": "报告期",
            "F005N": "占净资产比例",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "行业编码",
            "证监会行业名称",
            "报告期",
            "基金覆盖家数",
            "行业规模",
            "占净资产比例",
        ]
    ]
    temp_df["报告期"] = pd.to_datetime(temp_df["报告期"], errors="coerce").dt.date
    temp_df["基金覆盖家数"] = pd.to_numeric(temp_df["基金覆盖家数"], errors="coerce")
    temp_df["行业规模"] = pd.to_numeric(temp_df["行业规模"], errors="coerce")
    temp_df["占净资产比例"] = pd.to_numeric(temp_df["占净资产比例"], errors="coerce")
    return temp_df


def fund_report_asset_allocation_cninfo() -> pd.DataFrame:
    """
    巨潮资讯-数据中心-专题统计-基金报表-基金资产配置
    https://webapi.cninfo.com.cn/#/thematicStatistics
    :return: 基金资产配置
    :rtype: pandas.DataFrame
    """
    url = "https://webapi.cninfo.com.cn/api/sysapi/p_sysapi1114"
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_cninfo("cninfo.js")
    js_code.eval(js_content)
    mcode = js_code.call("getResCode1")
    headers = {
        "Accept": "*/*",
        "Accept-Enckey": mcode,
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "Origin": "https://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "https://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    r = requests.post(url, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.rename(
        columns={
            "F001N": "基金覆盖家数",
            "F008N": "现金货币类占净资产比例",
            "F007N": "债券固定收益类占净资产比例",
            "F006N": "股票权益类占净资产比例",
            "ENDDATE": "报告期",
            "F005N": "基金市场净资产规模",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "报告期",
            "基金覆盖家数",
            "股票权益类占净资产比例",
            "债券固定收益类占净资产比例",
            "现金货币类占净资产比例",
            "基金市场净资产规模",
        ]
    ]
    temp_df["报告期"] = pd.to_datetime(temp_df["报告期"], errors="coerce").dt.date
    temp_df["基金覆盖家数"] = pd.to_numeric(temp_df["基金覆盖家数"], errors="coerce")
    temp_df["股票权益类占净资产比例"] = pd.to_numeric(
        temp_df["股票权益类占净资产比例"], errors="coerce"
    )
    temp_df["债券固定收益类占净资产比例"] = pd.to_numeric(
        temp_df["债券固定收益类占净资产比例"], errors="coerce"
    )
    temp_df["现金货币类占净资产比例"] = pd.to_numeric(
        temp_df["现金货币类占净资产比例"], errors="coerce"
    )
    temp_df["基金市场净资产规模"] = pd.to_numeric(
        temp_df["基金市场净资产规模"], errors="coerce"
    )
    return temp_df


if __name__ == "__main__":
    fund_report_stock_cninfo_df = fund_report_stock_cninfo(date="20210630")
    print(fund_report_stock_cninfo_df)

    fund_report_industry_allocation_cninfo_df = fund_report_industry_allocation_cninfo(
        date="20210930"
    )
    print(fund_report_industry_allocation_cninfo_df)

    fund_report_asset_allocation_cninfo_df = fund_report_asset_allocation_cninfo()
    print(fund_report_asset_allocation_cninfo_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/8/20 16:13
Desc: 期货衍生指标配置文件
西本新干线-指数数据
"""
# xgx
symbol_dict = {
    '钢材指数': '65',
    '铁矿指数': '61',
    '焦炭指数': '64',
    '煤炭指数': '1002',
    '水泥指数': '1003',
    'FTZ指数': '1100',
    '钢铁行业PMI指数': '118',
    '生产指数': '119',
    '新订单指数': '120',
    '新出口订单指数': '121',
    '产成品库存指数': '122',
    '原材料库存指数': '123',
    '沪市终端线螺每周采购量监控': '74',
    '沪螺纹钢社会库存': '72',
    '国内螺纹钢社会库存量': '67',
    '国内线材社会库存量': '68',
    '国内主要城市热轧卷板库存': '69',
    '国内主要城市冷轧卷板库存': '70',
    '国内主要城市中厚板库存': '73',
    '全国主要钢材品种库存总量': '117',
    '热轧价格走势': '108',
    '冷轧价格走势': '109',
    '中板价格走势': '110',
    '型材价格走势': '111',
    '沪二级螺纹钢价格走势': '127',
    '螺纹钢主力合约收盘价格': '179',
    '铁矿石主力合约收盘价格': '180',
    '热轧板卷主力合约收盘价格': '181',
    '焦煤主力合约收盘价格': '182',
    '焦炭主力合约收盘价格': '183',
    '存款基准利率': '52',
    '贷款基准利率': '53',
    '存款准备金率': '105',
    '人民币新增贷款（亿元）': '174',
    '广义货币供应量增速（M2，%）': '175',
    '狭义货币供应量增速（M1，%)': '176',
    '上海大额银行承兑汇票(Ⅰ)': '129',
    '上海大额银行承兑汇票(Ⅱ)': '130',
    '上海大额银行承兑汇票(Ⅲ)': '131',
    '上海大额商业承兑汇票(Ⅰ)': '132',
    '上海大额商业承兑汇票(II)': '133',
    '上海大额商业承兑汇票(III)': '134',
    '重点企业粗钢日均产量（旬报）': '99',
    '重点企业钢材库存量（旬报）': '124',
    '国内月度粗钢日均产量': '159',
    '国内月度粗钢产量': '35',
    '国内月度钢材产量': '88',
    '国内月度螺纹钢产量': '40',
    '国内月度线材产量': '41',
    '国内月度热轧板卷产量': '114',
    '国内月度冷轧板卷产量': '115',
    '国内月度中厚板产量': '116',
    '国内月度生铁产量': '177',
    '国内月度焦炭产量': '37',
    '国内月度铁矿石原矿产量': '36',
    '国内月度铁矿石进口量': '42',
    '国内月度钢材出口量': '38',
    '国内月度钢材进口量': '39',
    '国内铁矿石港口存量': '43',
    '唐山地区钢坯库存量': '161',
    '印度矿港口库存': '100',
    '波罗的海干散货指数（BDI）': '77',
    '废钢价格走势': '78',
    '钢坯价格走势': '79',
    '钢材成本指数': '178',
    '铁矿石进口月度均价': '93',
    '巴西图巴朗-北仑铁矿海运价': '94',
    '西澳-北仑铁矿海运价': '95',
    '澳大利亚粉矿价格（56.5%，日照港）': '1006',
    '澳大利亚粉矿价格(61.5%青岛港，元/吨）': '106',
    '巴西粉矿价格（ 65% 日照港，元/吨）': '107',
    '62%铁矿石指数': '125',
    '63.5%印度粉矿外盘报价': '126',
    '国民生产总值季度增速（GDP）': '166',
    '居民消费物价指数（CPI）': '30',
    '工业生产者出厂价格指数（PPI）': '165',
    '制造业采购经理指数（PMI）': '104',
    '月度建筑安装工程投资额': '91',
    '月度固定资产投资额': '32',
    '月度房地产建设投资额': '34',
    '城填固定资产投资增速（累计值，%）': '171',
    '房地产开发投资增速（累计值，%）': '167',
    '土地购置面积同比增速（累计值，%）': '168',
    '房屋新开工面积同比增速（累计值，%）': '169',
    '商品房销售面积同比增速（累计值，%）': '170',
    '钢铁业固定资产投资增速（累计值，%）': '172',
    'CRU全球': '80',
    'CRU长材': '81',
    'CRU扁平材': '82',
    'CRU北美': '83',
    'CRU欧洲': '84',
    'CRU亚洲': '85',
    '全球粗钢月度产量（万吨）': '162',
    '全球粗钢日均产量（万吨）': '163',
    '全球粗钢产能利用率（%）': '164'}

xgx_code_url = "http://www.96369.net/Other/ValidateCode.aspx"
xgx_main_url = "http://www.96369.net/indices/{}"
xgx_headers = {
    "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3",
    "Accept-Encoding": "gzip, deflate",
    "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
    "Cache-Control": "no-cache",
    "Connection": "keep-alive",
    "Content-Length": "57",
    "Content-Type": "application/x-www-form-urlencoded",
    "Host": "www.96369.net",
    "Origin": "http://www.96369.net",
    "Pragma": "no-cache",
    "Referer": "http://www.96369.net/indices/67",
    "Upgrade-Insecure-Requests": "1",
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/77.0.3865.120 Safari/537.36"
}
xgx_short_headers = {
    "Accept": "image/webp,image/apng,image/*,*/*;q=0.8",
    "Accept-Encoding": "gzip, deflate",
    "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
    "Cache-Control": "no-cache",
    "Connection": "keep-alive",
    "Host": "www.96369.net",
    "Pragma": "no-cache",
    "Referer": "http://www.96369.net/indices/67",
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/77.0.3865.120 Safari/537.36"
}
# csa
csa_params_url = "https://www.jiaoyifamen.com/tools/nav/spread"
csa_url_spread = "https://www.jiaoyifamen.com/tools/future/spread/free"
csa_url_ratio = "https://www.jiaoyifamen.com/tools/future/valence/free"
csa_url_customize = "https://www.jiaoyifamen.com/tools/future/customize"

csa_payload = {
    "type1": "RB",
    "code1": "01",
    "type2": "RB",
    "code2": "05"
}



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2022/9/30 15:20
Desc: 参考汇率和结算汇率
深港通-港股通业务信息
深港通-港股通业务信息: https://www.szse.cn/szhk/hkbussiness/exchangerate/index.html
沪港通-港股通信息披露: http://www.sse.com.cn/services/hkexsc/disclo/ratios/
"""
from datetime import datetime

import pandas as pd
import requests


def stock_sgt_settlement_exchange_rate_szse() -> pd.DataFrame:
    """
    深港通-港股通业务信息-结算汇率
    https://www.szse.cn/szhk/hkbussiness/exchangerate/index.html
    :return: 结算汇率
    :rtype: pandas.DataFrame
    """
    url = "https://www.szse.cn/api/report/ShowReport"
    params = {
        'SHOWTYPE': 'xlsx',
        'CATALOGID': 'SGT_LSHL',
        'TABKEY': 'tab2',
        'random': '0.9184251620553985',
    }
    r = requests.get(url, params=params)
    import warnings
    with warnings.catch_warnings(record=True):
        warnings.simplefilter("always")
        temp_df = pd.read_excel(r.content, engine="openpyxl")
    temp_df.sort_values('适用日期', inplace=True, ignore_index=True)
    temp_df['适用日期'] = pd.to_datetime(temp_df['适用日期']).dt.date
    temp_df['买入结算汇兑比率'] = pd.to_numeric(temp_df['买入结算汇兑比率'], errors='coerce')
    temp_df['卖出结算汇兑比率'] = pd.to_numeric(temp_df['卖出结算汇兑比率'], errors='coerce')
    return temp_df


def stock_sgt_reference_exchange_rate_szse() -> pd.DataFrame:
    """
    深港通-港股通业务信息-参考汇率
    https://www.szse.cn/szhk/hkbussiness/exchangerate/index.html
    :return: 参考汇率
    :rtype: pandas.DataFrame
    """
    url = "https://www.szse.cn/api/report/ShowReport"
    params = {
        'SHOWTYPE': 'xlsx',
        'CATALOGID': 'SGT_LSHL',
        'TABKEY': 'tab1',
        'random': '0.9184251620553985',
    }
    r = requests.get(url, params=params)
    import warnings
    with warnings.catch_warnings(record=True):
        warnings.simplefilter("always")
        temp_df = pd.read_excel(r.content, engine="openpyxl")
    temp_df.sort_values('适用日期', inplace=True, ignore_index=True)
    temp_df['适用日期'] = pd.to_datetime(temp_df['适用日期']).dt.date
    temp_df['参考汇率买入价'] = pd.to_numeric(temp_df['参考汇率买入价'], errors='coerce')
    temp_df['参考汇率卖出价'] = pd.to_numeric(temp_df['参考汇率卖出价'], errors='coerce')
    return temp_df


def stock_sgt_reference_exchange_rate_sse() -> pd.DataFrame:
    """
    沪港通-港股通信息披露-参考汇率
    http://www.sse.com.cn/services/hkexsc/disclo/ratios/
    :return: 参考汇率
    :rtype: pandas.DataFrame
    """
    current_date = datetime.now().date().isoformat().replace("-", "")
    url = "http://query.sse.com.cn/commonSoaQuery.do"
    params = {
        'isPagination': 'true',
        'updateDate': '20120601',
        'updateDateEnd': current_date,
        'sqlId': 'FW_HGT_GGTHL',
        'pageHelp.cacheSize': '1',
        'pageHelp.pageSize': '10000',
        'pageHelp.pageNo': '1',
        'pageHelp.beginPage': '1',
        'pageHelp.endPage': '1',
        '_': '1664523262778',
    }
    headers = {
        'Host': 'query.sse.com.cn',
        'Referer': 'http://www.sse.com.cn/',
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36'
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json['result'])
    temp_df.rename(columns={
        'currencyType': "货币种类",
        'buyPrice': "参考汇率买入价",
        'updateDate': "-",
        'validDate': "适用日期",
        'sellPrice': "参考汇率卖出价"
    }, inplace=True)
    temp_df = temp_df[[
        "适用日期",
        "参考汇率买入价",
        "参考汇率卖出价",
        "货币种类",
    ]]
    temp_df.sort_values('适用日期', inplace=True, ignore_index=True)
    temp_df['适用日期'] = pd.to_datetime(temp_df['适用日期']).dt.date
    temp_df['参考汇率买入价'] = pd.to_numeric(temp_df['参考汇率买入价'], errors='coerce')
    temp_df['参考汇率卖出价'] = pd.to_numeric(temp_df['参考汇率卖出价'], errors='coerce')
    return temp_df


def stock_sgt_settlement_exchange_rate_sse() -> pd.DataFrame:
    """
    沪港通-港股通信息披露-结算汇兑
    http://www.sse.com.cn/services/hkexsc/disclo/ratios/
    :return: 结算汇兑比率
    :rtype: pandas.DataFrame
    """
    current_date = datetime.now().date().isoformat().replace("-", "")
    url = "http://query.sse.com.cn/commonSoaQuery.do"
    params = {
        'isPagination': 'true',
        'updateDate': '20120601',
        'updateDateEnd': current_date,
        'sqlId': 'FW_HGT_JSHDBL',
        'pageHelp.cacheSize': '1',
        'pageHelp.pageSize': '10000',
        'pageHelp.pageNo': '1',
        'pageHelp.beginPage': '1',
        'pageHelp.endPage': '1',
        '_': '1664523262778',
    }
    headers = {
        'Host': 'query.sse.com.cn',
        'Referer': 'http://www.sse.com.cn/',
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36'
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json['result'])
    temp_df.rename(columns={
        'currencyType': "货币种类",
        'buyPrice': "买入结算汇兑比率",
        'updateDate': "-",
        'validDate': "适用日期",
        'sellPrice': "卖出结算汇兑比率"
    }, inplace=True)
    temp_df = temp_df[[
        "适用日期",
        "买入结算汇兑比率",
        "卖出结算汇兑比率",
        "货币种类",
    ]]
    temp_df.sort_values('适用日期', inplace=True, ignore_index=True)
    temp_df['适用日期'] = pd.to_datetime(temp_df['适用日期']).dt.date
    temp_df['买入结算汇兑比率'] = pd.to_numeric(temp_df['买入结算汇兑比率'], errors='coerce')
    temp_df['卖出结算汇兑比率'] = pd.to_numeric(temp_df['卖出结算汇兑比率'], errors='coerce')
    return temp_df


if __name__ == "__main__":
    stock_sgt_settlement_exchange_rate_szse_df = stock_sgt_settlement_exchange_rate_szse()
    print(stock_sgt_settlement_exchange_rate_szse_df)

    stock_sgt_reference_exchange_rate_szse_df = stock_sgt_reference_exchange_rate_szse()
    print(stock_sgt_reference_exchange_rate_szse_df)

    stock_sgt_reference_exchange_rate_sse_df = stock_sgt_reference_exchange_rate_sse()
    print(stock_sgt_reference_exchange_rate_sse_df)

    stock_sgt_settlement_exchange_rate_sse_df = stock_sgt_settlement_exchange_rate_sse()
    print(stock_sgt_settlement_exchange_rate_sse_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/6/8 16:34
Desc: 巨潮资讯-首页-数据-预约披露
http://www.cninfo.com.cn/new/commonUrl?url=data/yypl
"""
import pandas as pd
import requests


def stock_report_disclosure(
    market: str = "沪深京", period: str = "2021年报"
) -> pd.DataFrame:
    """
    巨潮资讯-首页-数据-预约披露
    http://www.cninfo.com.cn/new/commonUrl?url=data/yypl
    :param market: choice of {"沪深京": "szsh", "深市": "sz", "深主板": "szmb", "中小板": "szsme", "创业板": "szcn", "沪市": "sh", "沪主板": "shmb", "科创板": "shkcp"}
    :type market: str
    :param period: 最近四期的财报
    :type period: str
    :return: 指定 market 和 period 的数据
    :rtype: pandas.DataFrame
    """
    market_map = {
        "沪深京": "szsh",
        "深市": "sz",
        "深主板": "szmb",
        "创业板": "szcn",
        "沪市": "sh",
        "沪主板": "shmb",
        "科创板": "shkcp",
        "北交所": "bj",
    }
    year = period[:4]
    period_map = {
        f"{year}一季": f"{year}-03-31",
        f"{year}半年报": f"{year}-06-30",
        f"{year}三季": f"{year}-09-30",
        f"{year}年报": f"{year}-12-31",
    }
    url = "http://www.cninfo.com.cn/new/information/getPrbookInfo"
    params = {
        "sectionTime": period_map[period],
        "firstTime": "",
        "lastTime": "",
        "market": market_map[market],
        "stockCode": "",
        "orderClos": "",
        "isDesc": "",
        "pagesize": "10000",
        "pagenum": "1",
    }
    r = requests.post(url, params=params)
    text_json = r.json()
    temp_df = pd.DataFrame(text_json["prbookinfos"])
    temp_df.columns = [
        "股票代码",
        "股票简称",
        "首次预约",
        "实际披露",
        "初次变更",
        "二次变更",
        "三次变更",
        "报告期",
        "-",
        "组织码",
    ]
    temp_df = temp_df[
        [
            "股票代码",
            "股票简称",
            "首次预约",
            "初次变更",
            "二次变更",
            "三次变更",
            "实际披露",
        ]
    ]
    temp_df["首次预约"] = pd.to_datetime(temp_df["首次预约"], errors="coerce").dt.date
    temp_df["初次变更"] = pd.to_datetime(temp_df["初次变更"], errors="coerce").dt.date
    temp_df["二次变更"] = pd.to_datetime(temp_df["二次变更"], errors="coerce").dt.date
    temp_df["三次变更"] = pd.to_datetime(temp_df["三次变更"], errors="coerce").dt.date
    temp_df["实际披露"] = pd.to_datetime(temp_df["实际披露"], errors="coerce").dt.date
    return temp_df


if __name__ == "__main__":
    stock_report_disclosure_df = stock_report_disclosure(market="沪深京", period="2022年报")
    print(stock_report_disclosure_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/8/05 15:30
Desc: 财新网-财新数据通
https://cxdata.caixin.com/pc/
"""

import pandas as pd
import requests


def stock_news_main_cx() -> pd.DataFrame:
    """
    财新网-财新数据通
    https://cxdata.caixin.com/pc/
    :return: 特定时间表示的字典
    :rtype: pandas.DataFrame
    """
    url = "https://cxdata.caixin.com/api/dataplus/sjtPc/jxNews"
    params = {
        "pageNum": "1",
        "pageSize": "20000",
        "showLabels": "true",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"]["data"])
    temp_df = temp_df[["tag", "summary", "intervalTime", "pubTime", "url"]]
    temp_df.columns = ["tag", "summary", "interval_time", "pub_time", "url"]
    temp_df["pub_time"] = pd.to_datetime(
        temp_df["pub_time"], errors="coerce", unit="ms"
    ).astype(str)
    return temp_df


if __name__ == "__main__":
    stock_news_main_cx_df = stock_news_main_cx()
    print(stock_news_main_cx_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2024/6/13 13:00
Desc: 东方财富网-数据中心-公告大全-沪深 A 股公告
https://data.eastmoney.com/notices/hsa/5.html
"""

import math

import pandas as pd
import requests

from byteapp_ak.utils.tqdm import get_tqdm


def stock_notice_report(symbol: str = "全部", date: str = "20220511") -> pd.DataFrame:
    """
    东方财富网-数据中心-公告大全-沪深京 A 股公告
    https://data.eastmoney.com/notices/hsa/5.html
    :param symbol: 报告类型; choice of {"全部", "重大事项", "财务报告", "融资公告", "风险提示", "资产重组", "信息变更", "持股变动"}
    :type symbol: str
    :param date: 制定日期
    :type date: str
    :return: 沪深京 A 股公告
    :rtype: pandas.DataFrame
    """
    url = "https://np-anotice-stock.eastmoney.com/api/security/ann"
    report_map = {
        "全部": "0",
        "财务报告": "1",
        "融资公告": "2",
        "风险提示": "3",
        "信息变更": "4",
        "重大事项": "5",
        "资产重组": "6",
        "持股变动": "7",
    }
    params = {
        "sr": "-1",
        "page_size": "100",
        "page_index": "1",
        "ann_type": "A",
        "client_source": "web",
        "f_node": report_map[symbol],
        "s_node": "0",
        "begin_time": "-".join([date[:4], date[4:6], date[6:]]),
        "end_time": "-".join([date[:4], date[4:6], date[6:]]),
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = math.ceil(data_json["data"]["total_hits"] / 100)
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, int(total_page) + 1), leave=False):
        params.update(
            {
                "page_index": page,
            }
        )
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["data"]["list"])
        temp_codes_df = pd.DataFrame(
            [item["codes"][0] for item in data_json["data"]["list"]]
        )
        try:
            temp_columns_df = pd.DataFrame(
                [item["columns"][0] for item in data_json["data"]["list"]]
            )
        except:  # noqa: E722
            continue
        del temp_df["codes"]
        del temp_df["columns"]
        temp_df = pd.concat(objs=[temp_df, temp_columns_df, temp_codes_df], axis=1)
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

    big_df.rename(
        columns={
            "art_code": "编码",
            "display_time": "-",
            "eiTime": "-",
            "notice_date": "公告日期",
            "title": "公告标题",
            "column_code": "-",
            "column_name": "公告类型",
            "ann_type": "-",
            "inner_code": "-",
            "market_code": "-",
            "short_name": "名称",
            "stock_code": "代码",
        },
        inplace=True,
    )
    url = "https://data.eastmoney.com/notices/detail/"
    big_df["网址"] = url + big_df["代码"] + "/" + big_df["编码"] + ".html"
    big_df = big_df[
        [
            "代码",
            "名称",
            "公告标题",
            "公告类型",
            "公告日期",
            "网址",
        ]
    ]
    big_df["公告日期"] = pd.to_datetime(big_df["公告日期"], errors="coerce").dt.date
    return big_df


if __name__ == "__main__":
    stock_notice_report_df = stock_notice_report(symbol="财务报告", date="20240612")
    print(stock_notice_report_df)

    item_list = [
        "全部",
        "财务报告",
        "融资公告",
        "风险提示",
        "信息变更",
        "重大事项",
        "资产重组",
        "持股变动",
    ]
    for temp_item in item_list:
        stock_notice_report_df = stock_notice_report(symbol=temp_item, date="20220511")
        print(stock_notice_report_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/4/25 17:20
Desc: 河北省空气质量预报信息发布系统
https://110.249.223.67/publish
每日 17 时发布
等级划分
1. 空气污染指数为0－50，空气质量级别为一级，空气质量状况属于优。此时，空气质量令人满意，基本无空气污染，各类人群可正常活动。
2. 空气污染指数为51－100，空气质量级别为二级，空气质量状况属于良。此时空气质量可接受，但某些污染物可能对极少数异常敏感人群健康有较弱影响，建议极少数异常敏感人群应减少户外活动。
3. 空气污染指数为101－150，空气质量级别为三级，空气质量状况属于轻度污染。此时，易感人群症状有轻度加剧，健康人群出现刺激症状。建议儿童、老年人及心脏病、呼吸系统疾病患者应减少长时间、高强度的户外锻炼。
4. 空气污染指数为151－200，空气质量级别为四级，空气质量状况属于中度污染。此时，进一步加剧易感人群症状，可能对健康人群心脏、呼吸系统有影响，建议疾病患者避免长时间、高强度的户外锻练，一般人群适量减少户外运动。
5. 空气污染指数为201－300，空气质量级别为五级，空气质量状况属于重度污染。此时，心脏病和肺病患者症状显著加剧，运动耐受力降低，健康人群普遍出现症状，建议儿童、老年人和心脏病、肺病患者应停留在室内，停止户外运动，一般人群减少户外运动。
6. 空气污染指数大于300，空气质量级别为六级，空气质量状况属于严重污染。此时，健康人群运动耐受力降低，有明显强烈症状，提前出现某些疾病，建议儿童、老年人和病人应当留在室内，避免体力消耗，一般人群应避免户外活动。
发布单位：河北省环境应急与重污染天气预警中心 技术支持：中国科学院大气物理研究所 中科三清科技有限公司
"""

from datetime import datetime

import pandas as pd
import requests
from tqdm import tqdm


def air_quality_hebei(symbol: str = "唐山市") -> pd.DataFrame:
    """
    河北省空气质量预报信息发布系统-空气质量预报, 未来 6 天
    https://110.249.223.67/publish/
    :param symbol: choice of {'石家庄市', '唐山市', '秦皇岛市', '邯郸市', '邢台市', '保定市', '张家口市', '承德市', '沧州市', '廊坊市', '衡水市', '辛集市', '定州市'}
    :type symbol: str
    :return: city = "", 返回所有地区的数据; city="唐山市", 返回唐山市的数据
    :rtype: pandas.DataFrame
    """
    url = (
        "http://110.249.223.67/server/api/CityPublishInfo/GetProvinceAndCityPublishData"
    )
    params = {"publishDate": f"{datetime.today().strftime('%Y-%m-%d')} 16:00:00"}
    r = requests.get(url, params=params)
    json_data = r.json()
    city_list = pd.DataFrame.from_dict(json_data["cityPublishDatas"], orient="columns")[
        "CityName"
    ].tolist()
    outer_df = pd.DataFrame()
    for i in tqdm(range(1, 7), leave=False):
        inner_df = pd.DataFrame(
            [item[f"Date{i}"] for item in json_data["cityPublishDatas"]],
            index=city_list,
        )
        outer_df = pd.concat([outer_df, inner_df])
    if symbol == "":
        temp_df = outer_df.reset_index()
        temp_df.columns = [
            "city",
            "date",
            "pollutant",
            "minAQI",
            "maxAQI",
            "level",
        ]
        temp_df["date"] = pd.to_datetime(temp_df["date"]).dt.date
        temp_df["minaqi"] = pd.to_numeric(temp_df["minaqi"])
        temp_df["maxaqi"] = pd.to_numeric(temp_df["maxaqi"])
        return temp_df
    else:
        temp_df = outer_df.reset_index()
        temp_df.columns = [
            "city",
            "date",
            "pollutant",
            "minaqi",
            "maxaqi",
            "level",
        ]
        temp_df["date"] = pd.to_datetime(temp_df["date"]).dt.date
        temp_df["minaqi"] = pd.to_numeric(temp_df["minaqi"])
        temp_df["maxaqi"] = pd.to_numeric(temp_df["maxaqi"])
        temp_df = temp_df[temp_df["city"] == symbol]
        temp_df.reset_index(inplace=True, drop=True)
        return temp_df


if __name__ == "__main__":
    air_quality_hebei_df = air_quality_hebei(symbol="定州市")
    print(air_quality_hebei_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2024/6/27 22:20
Desc: 上海证券交易所-产品-股票期权-期权风险指标
"""

import requests
import pandas as pd


def option_risk_indicator_sse(date: str = "20240626") -> pd.DataFrame:
    """
    上海证券交易所-产品-股票期权-期权风险指标
    http://www.sse.com.cn/assortment/options/risk/
    :param date: 日期; 20150209 开始
    :type date: str
    :return: 期权风险指标
    :rtype: pandas.DataFrame
    """
    url = "http://query.sse.com.cn/commonQuery.do"
    params = {
        "isPagination": "false",
        "trade_date": date,
        "sqlId": "SSE_ZQPZ_YSP_GGQQZSXT_YSHQ_QQFXZB_DATE_L",
        "contractSymbol": "",
        "_": "1652877575590",
    }
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Connection": "keep-alive",
        "Host": "query.sse.com.cn",
        "Pragma": "no-cache",
        "Referer": "http://www.sse.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/101.0.4951.67 Safari/537.36",
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"])
    temp_df = temp_df[
        [
            "TRADE_DATE",
            "SECURITY_ID",
            "CONTRACT_ID",
            "CONTRACT_SYMBOL",
            "DELTA_VALUE",
            "THETA_VALUE",
            "GAMMA_VALUE",
            "VEGA_VALUE",
            "RHO_VALUE",
            "IMPLC_VOLATLTY",
        ]
    ]
    temp_df["TRADE_DATE"] = pd.to_datetime(
        temp_df["TRADE_DATE"], errors="coerce"
    ).dt.date
    temp_df["DELTA_VALUE"] = pd.to_numeric(temp_df["DELTA_VALUE"], errors="coerce")
    temp_df["THETA_VALUE"] = pd.to_numeric(temp_df["THETA_VALUE"], errors="coerce")
    temp_df["GAMMA_VALUE"] = pd.to_numeric(temp_df["GAMMA_VALUE"], errors="coerce")
    temp_df["VEGA_VALUE"] = pd.to_numeric(temp_df["VEGA_VALUE"], errors="coerce")
    temp_df["RHO_VALUE"] = pd.to_numeric(temp_df["RHO_VALUE"], errors="coerce")
    temp_df["IMPLC_VOLATLTY"] = pd.to_numeric(
        temp_df["IMPLC_VOLATLTY"], errors="coerce"
    )
    return temp_df


if __name__ == "__main__":
    option_risk_indicator_sse_df = option_risk_indicator_sse(date="20240626")
    print(option_risk_indicator_sse_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/05/23 14:00
Desc: 个股新闻数据
https://so.eastmoney.com/news/s?keyword=%E4%B8%AD%E5%9B%BD%E4%BA%BA%E5%AF%BF&pageindex=1&searchrange=8192&sortfiled=4
"""

import json

import pandas as pd
import requests


def stock_news_em(symbol: str = "300059") -> pd.DataFrame:
    """
    东方财富-个股新闻-最近 100 条新闻
    https://so.eastmoney.com/news/s?keyword=%E4%B8%AD%E5%9B%BD%E4%BA%BA%E5%AF%BF&pageindex=1&searchrange=8192&sortfiled=4
    :param symbol: 股票代码
    :type symbol: str
    :return: 个股新闻
    :rtype: pandas.DataFrame
    """
    url = "http://search-api-web.eastmoney.com/search/jsonp"
    params = {
        "cb": "jQuery3510875346244069884_1668256937995",
        "param": '{"uid":"",'
        + f'"keyword":"{symbol}"'
        + ',"type":["cmsArticleWebOld"],"client":"web","clientType":"web","clientVersion":"curr",'
        '"param":{"cmsArticleWebOld":{"searchScope":"default","sort":"default","pageIndex":1,'
        '"pageSize":100,"preTag":"<em>","postTag":"</em>"}}}',
        "_": "1668256937996",
    }
    r = requests.get(url, params=params)
    data_text = r.text
    data_json = json.loads(
        data_text.strip("jQuery3510875346244069884_1668256937995(")[:-1]
    )
    temp_df = pd.DataFrame(data_json["result"]["cmsArticleWebOld"])
    temp_df.rename(
        columns={
            "date": "发布时间",
            "mediaName": "文章来源",
            "code": "-",
            "title": "新闻标题",
            "content": "新闻内容",
            "url": "新闻链接",
            "image": "-",
        },
        inplace=True,
    )
    temp_df["关键词"] = symbol
    temp_df = temp_df[
        [
            "关键词",
            "新闻标题",
            "新闻内容",
            "发布时间",
            "文章来源",
            "新闻链接",
        ]
    ]
    temp_df["新闻标题"] = (
        temp_df["新闻标题"]
        .str.replace(r"\(<em>", "", regex=True)
        .str.replace(r"</em>\)", "", regex=True)
    )
    temp_df["新闻标题"] = (
        temp_df["新闻标题"]
        .str.replace(r"<em>", "", regex=True)
        .str.replace(r"</em>", "", regex=True)
    )
    temp_df["新闻内容"] = (
        temp_df["新闻内容"]
        .str.replace(r"\(<em>", "", regex=True)
        .str.replace(r"</em>\)", "", regex=True)
    )
    temp_df["新闻内容"] = (
        temp_df["新闻内容"]
        .str.replace(r"<em>", "", regex=True)
        .str.replace(r"</em>", "", regex=True)
    )
    temp_df["新闻内容"] = temp_df["新闻内容"].str.replace(r"\u3000", "", regex=True)
    temp_df["新闻内容"] = temp_df["新闻内容"].str.replace(r"\r\n", " ", regex=True)
    return temp_df


if __name__ == "__main__":
    stock_news_em_df = stock_news_em(symbol="300059")
    print(stock_news_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/8/31 17:30
Desc: 新浪财经-外盘期货
https://finance.sina.com.cn/money/future/hf.html
"""

import time
from typing import Union, List

import pandas as pd
import requests
from bs4 import BeautifulSoup

from byteapp_ak.utils import demjson


def _get_real_name_list() -> list:
    """
    新浪-外盘期货所有品种的中文名称
    https://finance.sina.com.cn/money/future/hf.html
    :return: 外盘期货所有品种的中文名称
    :rtype: list
    """
    url = "https://finance.sina.com.cn/money/future/hf.html"
    r = requests.get(url)
    r.encoding = "gb2312"
    data_text = r.text
    need_text = data_text[
        data_text.find("var oHF_1 = ") + 12 : data_text.find("var oHF_2") - 2
    ].replace("\n\t", "")
    data_json = demjson.decode(need_text)
    name_list = [item[0].strip() for item in data_json.values()]
    return name_list


def futures_foreign_commodity_subscribe_exchange_symbol() -> list:
    """
    需要订阅的行情的代码
    https://finance.sina.com.cn/money/future/hf.html
    :return: 需要订阅的行情的代码
    :rtype: list
    """
    url = "https://finance.sina.com.cn/money/future/hf.html"
    r = requests.get(url)
    r.encoding = "gb2312"
    data_text = r.text
    data_json = demjson.decode(
        data_text[
            data_text.find("var oHF_1 = ") + 12 : data_text.find("var oHF_2 = ") - 2
        ]
    )
    code_list = list(data_json.keys())
    return code_list


def futures_hq_subscribe_exchange_symbol() -> pd.DataFrame:
    """
    将品种字典转化为 pandas.DataFrame
    https://finance.sina.com.cn/money/future/hf.html
    :return: 品种对应表
    :rtype: pandas.DataFrame
    """
    inner_dict = {
        "新加坡铁矿石": "FEF",
        "马棕油": "FCPO",
        "日橡胶": "RSS3",
        "美国原糖": "RS",
        "CME比特币期货": "BTC",
        "NYBOT-棉花": "CT",
        "LME镍3个月": "NID",
        "LME铅3个月": "PBD",
        "LME锡3个月": "SND",
        "LME锌3个月": "ZSD",
        "LME铝3个月": "AHD",
        "LME铜3个月": "CAD",
        "CBOT-黄豆": "S",
        "CBOT-小麦": "W",
        "CBOT-玉米": "C",
        "CBOT-黄豆油": "BO",
        "CBOT-黄豆粉": "SM",
        "日本橡胶": "TRB",
        "COMEX铜": "HG",
        "NYMEX天然气": "NG",
        "NYMEX原油": "CL",
        "COMEX白银": "SI",
        "COMEX黄金": "GC",
        "CME-瘦肉猪": "LHC",
        "布伦特原油": "OIL",
        "伦敦金": "XAU",
        "伦敦银": "XAG",
        "伦敦铂金": "XPT",
        "伦敦钯金": "XPD",
        "欧洲碳排放": "EUA",
    }
    temp_df = pd.DataFrame.from_dict(inner_dict, orient="index")
    temp_df.reset_index(inplace=True)
    temp_df.columns = ["symbol", "code"]
    return temp_df


def futures_foreign_commodity_realtime(symbol: Union[str, List[str]]) -> pd.DataFrame:
    """
    新浪-外盘期货-行情数据
    https://finance.sina.com.cn/money/future/hf.html
    :param symbol: 通过调用 ak.futures_hq_subscribe_exchange_symbol() 函数来获取
    :type symbol: list or str
    :return: 行情数据
    :rtype: pandas.DataFrame
    """
    if isinstance(symbol, list):
        payload = "?list=" + ",".join(["hf_" + item for item in symbol])
    else:
        symbol = symbol.split(",")
        payload = "?list=" + ",".join(["hf_" + item for item in symbol])
    url = "https://hq.sinajs.cn/" + payload
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate, br",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Connection": "keep-alive",
        "Host": "hq.sinajs.cn",
        "Pragma": "no-cache",
        "Referer": "https://finance.sina.com.cn/",
        "sec-ch-ua": '" Not;A Brand";v="99", "Google Chrome";v="97", "Chromium";v="97"',
        "sec-ch-ua-mobile": "?0",
        "sec-ch-ua-platform": '"Windows"',
        "Sec-Fetch-Dest": "script",
        "Sec-Fetch-Mode": "no-cors",
        "Sec-Fetch-Site": "cross-site",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/97.0.4692.71 Safari/537.36",
    }
    r = requests.get(url, headers=headers)
    data_text = r.text
    data_df = pd.DataFrame(
        [
            item.strip().split("=")[1].split(",")
            for item in data_text.split(";")
            if item.strip() != ""
        ]
    )
    data_df.iloc[:, 0] = data_df.iloc[:, 0].str.replace('"', "")
    data_df.iloc[:, -1] = data_df.iloc[:, -1].str.replace('"', "")

    # 处理伦敦金 XAU 的情况
    if len(data_df.columns) == 14:
        data_df["temp"] = None

    data_df.columns = [
        "current_price",
        "-",
        "bid",
        "ask",
        "high",
        "low",
        "time",
        "last_settle_price",
        "open",
        "hold",
        "-",
        "-",
        "date",
        "symbol",
        "current_price_rmb",
    ]
    temp_symbol_code_df = futures_hq_subscribe_exchange_symbol()
    temp_symbol_code_dict = dict(
        zip(temp_symbol_code_df["code"], temp_symbol_code_df["symbol"])
    )
    data_df["symbol"] = [temp_symbol_code_dict[subscribe] for subscribe in symbol]
    data_df = data_df[
        [
            "symbol",
            "current_price",
            "current_price_rmb",
            "bid",
            "ask",
            "high",
            "low",
            "time",
            "last_settle_price",
            "open",
            "hold",
            "date",
        ]
    ]
    data_df.columns = [
        "名称",
        "最新价",
        "人民币报价",
        "买价",
        "卖价",
        "最高价",
        "最低价",
        "行情时间",
        "昨日结算价",
        "开盘价",
        "持仓量",
        "日期",
    ]
    data_df.dropna(how="all", inplace=True)
    data_df["最新价"] = pd.to_numeric(data_df["最新价"], errors="coerce")
    data_df["人民币报价"] = pd.to_numeric(data_df["人民币报价"], errors="coerce")
    data_df["买价"] = pd.to_numeric(data_df["买价"], errors="coerce")
    data_df["卖价"] = pd.to_numeric(data_df["卖价"], errors="coerce")
    data_df["最高价"] = pd.to_numeric(data_df["最高价"], errors="coerce")
    data_df["最低价"] = pd.to_numeric(data_df["最低价"], errors="coerce")
    data_df["昨日结算价"] = pd.to_numeric(data_df["昨日结算价"], errors="coerce")
    data_df["开盘价"] = pd.to_numeric(data_df["开盘价"], errors="coerce")
    data_df["持仓量"] = pd.to_numeric(data_df["持仓量"], errors="coerce")
    data_df["涨跌额"] = data_df["最新价"] - data_df["昨日结算价"]
    data_df["涨跌幅"] = (
        (data_df["最新价"] - data_df["昨日结算价"]) / data_df["昨日结算价"] * 100
    )
    data_df = data_df[
        [
            "名称",
            "最新价",
            "人民币报价",
            "涨跌额",
            "涨跌幅",
            "开盘价",
            "最高价",
            "最低价",
            "昨日结算价",
            "持仓量",
            "买价",
            "卖价",
            "行情时间",
            "日期",
        ]
    ]

    # 获取转换比例数据
    url = "https://finance.sina.com.cn/money/future/hf.html"
    r = requests.get(url)
    r.encoding = "utf-8"
    soup = BeautifulSoup(r.text, features="lxml")
    data_text = soup.find_all(name="script", attrs={"type": "text/javascript"})[
        -2
    ].string.strip()
    raw_text = data_text[data_text.find("oHF_1 = ") : data_text.find("oHF_2")]
    need_text = raw_text[raw_text.find("{") : raw_text.rfind("}") + 1]
    data_json = demjson.decode(need_text)
    price_mul = pd.DataFrame(
        [
            [item[0] for item in data_json.values()],
            [item[1][0] for item in data_json.values()],
        ]
    ).T
    price_mul.columns = ["symbol", "price"]
    price_mul = price_mul[price_mul["symbol"].isin(data_df["名称"])]
    price_mul.reset_index(inplace=True, drop=True)
    price_mul["price"] = pd.to_numeric(price_mul["price"], errors="coerce")

    # 获取汇率数据
    url = "https://hq.sinajs.cn/?list=USDCNY"
    r = requests.get(url, headers=headers)
    data_text = r.text
    usd_rmb = float(
        data_text[data_text.find('"') + 1 : data_text.find(",美元人民币")].split(",")[
            -1
        ]
    )

    # 计算人民币报价
    data_df["最新价"] = pd.to_numeric(data_df["最新价"], errors="coerce")
    data_df["人民币报价"] = data_df["最新价"] * price_mul["price"] * float(usd_rmb)
    data_df.dropna(thresh=4, inplace=True)
    return data_df


if __name__ == "__main__":
    futures_hq_subscribe_exchange_symbol_df = futures_hq_subscribe_exchange_symbol()
    print(futures_hq_subscribe_exchange_symbol_df)

    print("开始接收实时行情, 每秒刷新一次")
    subscribes = futures_foreign_commodity_subscribe_exchange_symbol()

    futures_foreign_commodity_realtime_df = futures_foreign_commodity_realtime(
        symbol="CT,NID"
    )
    print(futures_foreign_commodity_realtime_df)

    futures_foreign_commodity_realtime_df = futures_foreign_commodity_realtime(
        symbol=["XAU"]
    )
    print(futures_foreign_commodity_realtime_df)

    while True:
        futures_foreign_commodity_realtime_df = futures_foreign_commodity_realtime(
            symbol=subscribes
        )
        print(futures_foreign_commodity_realtime_df)
        time.sleep(3)
